@@ -16,8 +16,10 @@ package org.apache.sedona.common;
 import com.google.common.geometry.S2CellId;
 import com.uber.h3core.exceptions.H3Exception;
 
+import com.uber.h3core.util.LatLng;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.sedona.common.geometryObjects.Circle;
+import org.apache.sedona.common.sphere.Spheroid;
 import org.apache.sedona.common.subDivide.GeometrySubDivider;
 import org.apache.sedona.common.utils.GeomUtils;
 import org.apache.sedona.common.utils.GeometryGeoHashEncoder;
@@ -25,19 +27,9 @@ import org.apache.sedona.common.utils.GeometrySplitter;
 import org.apache.sedona.common.utils.H3Utils;
 import org.apache.sedona.common.utils.S2Utils;
 import org.locationtech.jts.algorithm.MinimumBoundingCircle;
+import org.locationtech.jts.algorithm.Orientation;
 import org.locationtech.jts.algorithm.hull.ConcaveHull;
-import org.locationtech.jts.geom.Coordinate;
-import org.locationtech.jts.geom.Geometry;
-import org.locationtech.jts.geom.GeometryCollection;
-import org.locationtech.jts.geom.GeometryFactory;
-import org.locationtech.jts.geom.LineString;
-import org.locationtech.jts.geom.LinearRing;
-import org.locationtech.jts.geom.MultiLineString;
-import org.locationtech.jts.geom.MultiPoint;
-import org.locationtech.jts.geom.MultiPolygon;
-import org.locationtech.jts.geom.Point;
-import org.locationtech.jts.geom.Polygon;
-import org.locationtech.jts.geom.PrecisionModel;
+import org.locationtech.jts.geom.*;
 import org.locationtech.jts.geom.impl.CoordinateArraySequence;
 import org.locationtech.jts.geom.util.GeometryFixer;
 import org.locationtech.jts.io.gml2.GMLWriter;
@@ -48,22 +40,20 @@ import org.locationtech.jts.operation.buffer.BufferParameters;
 import org.locationtech.jts.operation.distance.DistanceOp;
 import org.locationtech.jts.operation.distance3d.Distance3DOp;
 import org.locationtech.jts.operation.linemerge.LineMerger;
+import org.locationtech.jts.operation.overlay.snap.GeometrySnapper;
+import org.locationtech.jts.operation.polygonize.Polygonizer;
 import org.locationtech.jts.operation.valid.IsSimpleOp;
 import org.locationtech.jts.operation.valid.IsValidOp;
 import org.locationtech.jts.operation.valid.TopologyValidationError;
 import org.locationtech.jts.precision.GeometryPrecisionReducer;
 import org.locationtech.jts.simplify.TopologyPreservingSimplifier;
 import org.wololo.jts2geojson.GeoJSONWriter;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.HashSet;
-import java.util.List;
-import java.util.Set;
-import java.util.LinkedHashSet;
-import java.util.Collection;
+
+import java.util.*;
 import java.util.stream.Collectors;
 
 import static com.google.common.geometry.S2.DBL_EPSILON;
+import static org.apache.sedona.common.FunctionsGeoTools.bufferSpheroid;
 
 
 public class Functions {
@@ -97,25 +87,40 @@ public class Functions {
     }
 
     public static Geometry buffer(Geometry geometry, double radius) {
-        return buffer(geometry, radius, "");
+        return buffer(geometry, radius, false, "");
     }
 
-    public static Geometry buffer(Geometry geometry, double radius, String params) {
-        if (params.isEmpty()) {
-            return BufferOp.bufferOp(geometry, radius);
-        }
+    public static Geometry buffer(Geometry geometry, double radius, boolean useSpheroid) {
+        return buffer(geometry, radius, useSpheroid, "");
+    }
 
-        BufferParameters bufferParameters = parseBufferParams(params);
+    public static Geometry buffer(Geometry geometry, double radius, boolean useSpheroid, String params) throws IllegalArgumentException {
+        BufferParameters bufferParameters = new BufferParameters();
 
-        // convert the sign to the appropriate direction
-        // left - radius should be positive
-        // right - radius should be negative
-        if (bufferParameters.isSingleSided() &&
-                (params.toLowerCase().contains("left") && radius < 0 || params.toLowerCase().contains("right") && radius > 0)) {
+        // Processing parameters
+        if (!params.isEmpty()) {
+            bufferParameters = parseBufferParams(params);
+
+            // convert the sign to the appropriate direction
+            // left - radius should be positive
+            // right - radius should be negative
+            if (bufferParameters.isSingleSided() &&
+                    (params.toLowerCase().contains("left") && radius < 0 || params.toLowerCase().contains("right") && radius > 0)) {
                 radius = -radius;
+            }
         }
 
-        return BufferOp.bufferOp(geometry, radius, bufferParameters);
+        if (useSpheroid) {
+            // Spheroidal buffering logic
+            try {
+                return bufferSpheroid(geometry, radius, bufferParameters);
+            } catch (RuntimeException e) {
+                throw new RuntimeException("Error processing spheroidal buffer", e);
+            }
+        } else {
+            // Existing planar buffer logic with params handling
+            return BufferOp.bufferOp(geometry, radius, bufferParameters);
+        }
     }
 
     private static BufferParameters parseBufferParams(String params) {
@@ -196,23 +201,170 @@ public class Functions {
         return bufferParameters;
     }
 
-    public static Geometry envelope(Geometry geometry) {
+    public static int bestSRID(Geometry geometry) throws IllegalArgumentException {
+        // Shift longitudes if geometry crosses dateline
+        if (crossesDateLine(geometry)) {
+            shiftLongitude(geometry);
+        }
+
+        // Check envelope
+        Envelope envelope = geometry.getEnvelopeInternal();
+        if (envelope.isNull()) return Spheroid.EPSG_WORLD_MERCATOR; // Fallback EPSG
+
+        // Calculate the center of the envelope
+        double centerX = (envelope.getMinX() + envelope.getMaxX()) / 2.0; // centroid.getX();
+        double centerY = (envelope.getMinY() + envelope.getMaxY()) / 2.0; // centroid.getY();
+
+        // Calculate angular width and height
+        Double xwidth = Spheroid.angularWidth(envelope);
+        Double ywidth = Spheroid.angularHeight(envelope);
+        if (xwidth.isNaN() | ywidth.isNaN()) {
+            throw new IllegalArgumentException("Only lon/lat coordinate systems are supported by ST_BestSRID");
+        }
+
+        // Prioritize polar regions for Lambert Azimuthal Equal Area projection
+        if (centerY >= 70.0 && ywidth < 45.0) return Spheroid.EPSG_NORTH_LAMBERT;
+        if (centerY <= -70.0 && ywidth < 45.0) return Spheroid.EPSG_SOUTH_LAMBERT;
+
+        // Check for UTM zones
+        if (xwidth < 6.0) {
+            int zone;
+            if (centerX == -180.0 || centerX == 180.0) {
+                zone = 59; // UTM zone 60
+            } else {
+                zone = (int) Math.floor((centerX + 180.0) / 6.0);
+                zone = (zone > 59) ? zone - 60 : zone;
+            }
+            return (centerY < 0.0) ? Spheroid.EPSG_SOUTH_UTM_START + zone : Spheroid.EPSG_NORTH_UTM_START + zone;
+        }
+
+        // Default fallback to Mercator projection
+        return Spheroid.EPSG_WORLD_MERCATOR;
+    }
+
+    /**
+     * Corrects the longitudes of a geometry to be within the -180 to 180 range.
+     * This method modifies the original geometry.
+     *
+     * @param geometry The geometry to be corrected.
+     */
+    public static void normalizeLongitude(Geometry geometry) {
+        if (geometry == null || geometry.isEmpty()) {
+            return;
+        }
+
+        geometry.apply(new CoordinateSequenceFilter() {
+            @Override
+            public void filter(CoordinateSequence seq, int i) {
+                double x = seq.getX(i);
+                // Normalize the longitude to be within -180 to 180 range
+                while (x > 180) x -= 360;
+                while (x < -180) x += 360;
+                seq.setOrdinate(i, CoordinateSequence.X, x);
+            }
+
+            @Override
+            public boolean isDone() {
+                return false; // Continue processing until all coordinates are processed
+            }
+
+            @Override
+            public boolean isGeometryChanged() {
+                return true; // The geometry is changed as we are modifying the coordinates
+            }
+        });
+
+        geometry.geometryChanged(); // Notify the geometry that its coordinates have been changed
+    }
+
+    /**
+     * Checks if a geometry crosses the International Date Line.
+     *
+     * @param geometry The geometry to check.
+     * @return True if the geometry crosses the Date Line, false otherwise.
+     */
+    public static boolean crossesDateLine(Geometry geometry) {
+        if (geometry == null || geometry.isEmpty()) {
+            return false;
+        }
+
+        CoordinateSequenceFilter filter = new CoordinateSequenceFilter() {
+            private Coordinate previous = null;
+            private boolean crossesDateLine = false;
+
+            @Override
+            public void filter(CoordinateSequence seq, int i) {
+                if (i == 0) {
+                    previous = seq.getCoordinateCopy(i);
+                    return;
+                }
+
+                Coordinate current = seq.getCoordinateCopy(i);
+                if (Math.abs(current.x - previous.x) > 180) {
+                    crossesDateLine = true;
+                }
+
+                previous = current;
+            }
+
+            @Override
+            public boolean isDone() {
+                return crossesDateLine;
+            }
+
+            @Override
+            public boolean isGeometryChanged() {
+                return false;
+            }
+        };
+
+        geometry.apply(filter);
+        return filter.isDone();
+    }
+
+    public static Geometry shiftLongitude (Geometry geometry){
+        geometry.apply(new CoordinateSequenceFilter() {
+            @Override
+            public void filter(CoordinateSequence seq, int i) {
+                double newX = seq.getX(i);
+                if (newX < 0) {
+                    newX += 360;
+                } else if (newX > 180) {
+                    newX -= 360;
+                }
+                seq.setOrdinate(i, CoordinateSequence.X, newX);
+            }
+
+            @Override
+            public boolean isDone() {
+                return false;
+            }
+
+            @Override
+            public boolean isGeometryChanged() {
+                return true;
+            }
+        });
+        return geometry;
+    }
+
+    public static Geometry envelope (Geometry geometry){
         return geometry.getEnvelope();
     }
 
-    public static double distance(Geometry left, Geometry right) {
+    public static double distance (Geometry left, Geometry right){
         return left.distance(right);
     }
 
-    public static double distance3d(Geometry left, Geometry right) {
+    public static double distance3d (Geometry left, Geometry right){
         return new Distance3DOp(left, right).distance();
     }
 
-    public static double length(Geometry geometry) {
+    public static double length (Geometry geometry){
         return geometry.getLength();
     }
 
-    public static Geometry normalize(Geometry geometry) {
+    public static Geometry normalize (Geometry geometry){
         geometry.normalize();
         return geometry;
     }
@@ -655,6 +807,92 @@ public class Functions {
         return GEOMETRY_FACTORY.createPoint(interPoint);
     }
 
+    /**
+     * Forces a Polygon/MultiPolygon to use clockwise orientation for the exterior ring and a counter-clockwise for the interior ring(s).
+     * @param geom
+     * @return a clockwise orientated (Multi)Polygon
+     */
+    public static Geometry forcePolygonCW(Geometry geom) {
+        if (isPolygonCW(geom)) {
+            return geom;
+        }
+
+        if (geom instanceof Polygon) {
+            return transformCW((Polygon) geom);
+
+        } else if (geom instanceof MultiPolygon) {
+            List<Polygon> polygons = new ArrayList<>();
+            for (int i = 0; i < geom.getNumGeometries(); i++) {
+                Polygon polygon = (Polygon) geom.getGeometryN(i);
+                polygons.add((Polygon) transformCW(polygon));
+            }
+
+            return new GeometryFactory().createMultiPolygon(polygons.toArray(new Polygon[0]));
+
+        }
+        // Non-polygonal geometries are returned unchanged
+        return geom;
+    }
+
+    private static Geometry transformCW(Polygon polygon) {
+        LinearRing exteriorRing = polygon.getExteriorRing();
+        LinearRing exteriorRingEnforced = transformCW(exteriorRing, true);
+
+        List<LinearRing> interiorRings = new ArrayList<>();
+        for (int i = 0; i < polygon.getNumInteriorRing(); i++) {
+            interiorRings.add(transformCW(polygon.getInteriorRingN(i), false));
+        }
+
+        return new GeometryFactory(polygon.getPrecisionModel(), polygon.getSRID())
+                .createPolygon(exteriorRingEnforced, interiorRings.toArray(new LinearRing[0]));
+    }
+
+    private static LinearRing transformCW(LinearRing ring, boolean isExteriorRing) {
+        boolean isRingClockwise = !Orientation.isCCW(ring.getCoordinateSequence());
+
+        LinearRing enforcedRing;
+        if (isExteriorRing) {
+            enforcedRing = isRingClockwise ? (LinearRing) ring.copy() : ring.reverse();
+        } else {
+            enforcedRing = isRingClockwise ? ring.reverse() : (LinearRing) ring.copy();
+        }
+        return enforcedRing;
+    }
+
+    /**
+     * This function accepts Polygon and MultiPolygon, if any other type is provided then it will return false.
+     * If the exterior ring is clockwise and the interior ring(s) are counter-clockwise then returns true, otherwise false.
+     * @param geom Polygon or MultiPolygon
+     * @return
+     */
+    public static boolean isPolygonCW(Geometry geom) {
+        if (geom instanceof MultiPolygon) {
+            MultiPolygon multiPolygon = (MultiPolygon) geom;
+
+            boolean arePolygonsCW = checkIfPolygonCW((Polygon) multiPolygon.getGeometryN(0));
+            for (int i = 1; i < multiPolygon.getNumGeometries(); i++) {
+                arePolygonsCW = arePolygonsCW && checkIfPolygonCW((Polygon) multiPolygon.getGeometryN(i));
+            }
+            return arePolygonsCW;
+        } else if (geom instanceof Polygon) {
+            return checkIfPolygonCW((Polygon) geom);
+        }
+        // False for remaining geometry types
+        return false;
+    }
+
+    private static boolean checkIfPolygonCW(Polygon geom) {
+        LinearRing exteriorRing = geom.getExteriorRing();
+        boolean isExteriorRingCW = !Orientation.isCCW(exteriorRing.getCoordinateSequence());
+
+        boolean isInteriorRingCW = Orientation.isCCW(geom.getInteriorRingN(0).getCoordinateSequence());
+        for (int i = 1; i < geom.getNumInteriorRing(); i++) {
+            isInteriorRingCW = isInteriorRingCW && Orientation.isCCW(geom.getInteriorRingN(i).getCoordinateSequence());
+        }
+
+        return isExteriorRingCW && isInteriorRingCW;
+    }
+
     public static double lineLocatePoint(Geometry geom, Geometry point)
     {
         double length = geom.getLength();
@@ -662,6 +900,92 @@ public class Functions {
         return indexedLine.indexOf(point.getCoordinate()) / length;
     }
 
+    /**
+     * Forces a Polygon/MultiPolygon to use counter-clockwise orientation for the exterior ring and a clockwise for the interior ring(s).
+     * @param geom
+     * @return a counter-clockwise orientated (Multi)Polygon
+     */
+    public static Geometry forcePolygonCCW(Geometry geom) {
+        if (isPolygonCCW(geom)) {
+            return geom;
+        }
+
+        if (geom instanceof Polygon) {
+            return transformCCW((Polygon) geom);
+
+        } else if (geom instanceof MultiPolygon) {
+            List<Polygon> polygons = new ArrayList<>();
+            for (int i = 0; i < geom.getNumGeometries(); i++) {
+                Polygon polygon = (Polygon) geom.getGeometryN(i);
+                polygons.add((Polygon) transformCCW(polygon));
+            }
+
+            return new GeometryFactory().createMultiPolygon(polygons.toArray(new Polygon[0]));
+
+        }
+        // Non-polygonal geometries are returned unchanged
+        return geom;
+    }
+
+    private static Geometry transformCCW(Polygon polygon) {
+        LinearRing exteriorRing = polygon.getExteriorRing();
+        LinearRing exteriorRingEnforced = transformCCW(exteriorRing, true);
+
+        List<LinearRing> interiorRings = new ArrayList<>();
+        for (int i = 0; i < polygon.getNumInteriorRing(); i++) {
+            interiorRings.add(transformCCW(polygon.getInteriorRingN(i), false));
+        }
+
+        return new GeometryFactory(polygon.getPrecisionModel(), polygon.getSRID())
+                .createPolygon(exteriorRingEnforced, interiorRings.toArray(new LinearRing[0]));
+    }
+
+    private static LinearRing transformCCW(LinearRing ring, boolean isExteriorRing) {
+        boolean isRingCounterClockwise = Orientation.isCCW(ring.getCoordinateSequence());
+
+        LinearRing enforcedRing;
+        if (isExteriorRing) {
+            enforcedRing = isRingCounterClockwise ? (LinearRing) ring.copy() : ring.reverse();
+        } else {
+            enforcedRing = isRingCounterClockwise ? ring.reverse() : (LinearRing) ring.copy();
+        }
+        return enforcedRing;
+    }
+
+    /**
+     * This function accepts Polygon and MultiPolygon, if any other type is provided then it will return false.
+     * If the exterior ring is counter-clockwise and the interior ring(s) are clockwise then returns true, otherwise false.
+     * @param geom Polygon or MultiPolygon
+     * @return
+     */
+    public static boolean isPolygonCCW(Geometry geom) {
+        if (geom instanceof MultiPolygon) {
+            MultiPolygon multiPolygon = (MultiPolygon) geom;
+
+            boolean arePolygonsCCW = checkIfPolygonCCW(((Polygon) multiPolygon.getGeometryN(0)));
+            for (int i = 1; i < multiPolygon.getNumGeometries(); i++) {
+                arePolygonsCCW = arePolygonsCCW && checkIfPolygonCCW((Polygon) multiPolygon.getGeometryN(i));
+            }
+            return arePolygonsCCW;
+        } else if (geom instanceof Polygon) {
+            return checkIfPolygonCCW((Polygon) geom);
+        }
+        // False for remaining geometry types
+        return false;
+    }
+
+    private static boolean checkIfPolygonCCW(Polygon geom) {
+        LinearRing exteriorRing = geom.getExteriorRing();
+        boolean isExteriorRingCCW = Orientation.isCCW(exteriorRing.getCoordinateSequence());
+
+        boolean isInteriorRingCCW = !Orientation.isCCW(geom.getInteriorRingN(0).getCoordinateSequence());
+        for (int i = 1; i < geom.getNumInteriorRing(); i++) {
+            isInteriorRingCCW = isInteriorRingCCW && !Orientation.isCCW(geom.getInteriorRingN(i).getCoordinateSequence());
+        }
+
+        return isExteriorRingCCW && isInteriorRingCCW;
+    }
+
     public static Geometry difference(Geometry leftGeometry, Geometry rightGeometry) {
         boolean isIntersects = leftGeometry.intersects(rightGeometry);
         if (!isIntersects) {
@@ -707,6 +1031,16 @@ public class Functions {
         return S2Utils.roundCellsToSameLevel(new ArrayList<>(cellIds), level).stream().map(S2CellId::id).collect(Collectors.toList()).toArray(new Long[cellIds.size()]);
     }
 
+    /**
+     *
+     * @param cellIds array of cell ids
+     * @return An array of polygons for the cell ids
+     */
+    public static Geometry[] s2ToGeom(long[] cellIds) {
+        List<S2CellId> s2CellObjs = Arrays.stream(cellIds).mapToObj(S2CellId::new).collect(Collectors.toList());
+        return s2CellObjs.stream().map(S2Utils::toJTSPolygon).toArray(Polygon[]::new);
+    }
+
     /**
      * cover the geometry with H3 cells
      * @param input the input geometry
@@ -777,28 +1111,37 @@ public class Functions {
     }
 
     /**
-     * get the neighbor cells of the input cell by h3.gridDisk function
+     * gets the polygon for each h3 index provided
      * @param cells: the set of cells
-     * @return Multiple Polygons reversed
+     * @return An Array of Polygons reversed
      */
-    public static Geometry h3ToGeom(long[] cells) {
+    public static Geometry[] h3ToGeom(long[] cells) {
         GeometryFactory geomFactory = new GeometryFactory();
-        Collection<Long> h3 = Arrays.stream(cells).boxed().collect(Collectors.toList());
-        return geomFactory.createMultiPolygon(
-                H3Utils.h3.cellsToMultiPolygon(h3, true).stream().map(
-                        shellHoles -> {
-                            List<LinearRing> rings = shellHoles.stream().map(
-                                    shell -> geomFactory.createLinearRing(shell.stream().map(latLng -> new Coordinate(latLng.lng, latLng.lat)).toArray(Coordinate[]::new))
-                            ).collect(Collectors.toList());
-                            LinearRing shell = rings.remove(0);
-                            if (rings.isEmpty()) {
-                                return geomFactory.createPolygon(shell);
-                            } else {
-                                return geomFactory.createPolygon(shell, rings.toArray(new LinearRing[0]));
-                            }
-                        }
-                ).toArray(Polygon[]::new)
-        );
+        List<Long> h3 = Arrays.stream(cells).boxed().collect(Collectors.toList());
+        List<Polygon> polygons = new ArrayList<>();
+
+        for (int j = 0; j < h3.size(); j++) {
+            for (List<List<LatLng>> shellHoles : H3Utils.h3.cellsToMultiPolygon(Collections.singleton(h3.get(j)), true)) {
+                List<LinearRing> rings = new ArrayList<>();
+                for (List<LatLng> shell : shellHoles) {
+                    Coordinate[] coordinates = new Coordinate[shell.size()];
+                    for (int i = 0; i < shell.size(); i++) {
+                        LatLng latLng = shell.get(i);
+                        coordinates[i] = new Coordinate(latLng.lng, latLng.lat);
+                    }
+                    LinearRing ring = geomFactory.createLinearRing(coordinates);
+                    rings.add(ring);
+                }
+
+                LinearRing shell = rings.remove(0);
+                if (rings.isEmpty()) {
+                    polygons.add(geomFactory.createPolygon(shell));
+                } else {
+                    polygons.add(geomFactory.createPolygon(shell, rings.toArray(new LinearRing[0])));
+                }
+            }
+        }
+        return polygons.toArray(new Polygon[0]);
     }
 
     // create static function named simplifyPreserveTopology
@@ -859,6 +1202,10 @@ public class Functions {
         return leftGeom.union(rightGeom);
     }
 
+    public static Geometry union(Geometry[] geoms) {
+        return GEOMETRY_FACTORY.createGeometryCollection(geoms).union();
+    }
+
     public static Geometry createMultiGeometryFromOneElement(Geometry geometry) {
         if (geometry instanceof Circle) {
             return GEOMETRY_FACTORY.createGeometryCollection(new Circle[] {(Circle) geometry});
@@ -879,6 +1226,11 @@ public class Functions {
         return GeometrySubDivider.subDivide(geometry, maxVertices);
     }
 
+    public static Geometry snap(Geometry input, Geometry reference, double tolerance) {
+        GeometrySnapper snapper = new GeometrySnapper(input);
+        return snapper.snapTo(reference, tolerance);
+    }
+
     public static Geometry makeLine(Geometry geom1, Geometry geom2) {
         Geometry[] geoms = new Geometry[]{geom1, geom2};
         return makeLine(geoms);
@@ -1310,4 +1662,31 @@ public class Functions {
             return error.toString();
         }
     }
+
+    /**
+     * This method takes a GeometryCollection and returns another GeometryCollection
+     * containing the polygons formed by the linework of a set of geometries.
+     * @param geometry A collection of Geometry objects that should contain only linestrings.
+     * @return A GeometryCollection containing the resultant polygons.
+     */
+    public static Geometry polygonize(Geometry geometry) {
+        if (geometry == null || geometry.isEmpty()) {
+            return GEOMETRY_FACTORY.createGeometryCollection(null);
+        }
+
+        if (geometry instanceof GeometryCollection) {
+            Polygonizer polygonizer = new Polygonizer();
+
+            for (int i = 0; i < geometry.getNumGeometries(); i++) {
+                polygonizer.add(geometry.getGeometryN(i));
+            }
+
+            Collection polygons = polygonizer.getPolygons();
+            Geometry[] polyArray = (Geometry[]) polygons.toArray(new Geometry[0]);
+
+            return GEOMETRY_FACTORY.createGeometryCollection(polyArray);
+        } else {
+            return GEOMETRY_FACTORY.createGeometryCollection(null);
+        }
+    }
 }
