@@ -19,6 +19,7 @@
 package org.apache.sedona.common.raster;
 
 import org.apache.sedona.common.Constructors;
+import org.apache.sedona.common.raster.serde.Serde;
 import org.geotools.coverage.grid.GridCoverage2D;
 import org.junit.Test;
 import org.locationtech.jts.geom.Geometry;
@@ -35,6 +36,7 @@ import java.util.Map;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 
+import static org.apache.sedona.common.raster.RasterBandEditors.rasterUnion;
 import static org.junit.Assert.*;
 
 public class RasterBandEditorsTest extends RasterTestBase{
@@ -64,15 +66,40 @@ public class RasterBandEditorsTest extends RasterTestBase{
         assertEquals(expected, actual);
     }
 
+    @Test
+    public void testGetSummaryStats() throws IOException {
+        GridCoverage2D raster = rasterFromGeoTiff(resourceFolder + "raster/raster_with_no_data/test5.tiff");
+        raster = RasterBandEditors.setBandNoDataValue(raster, 1, 10.0, true);
+
+        // Test single output
+        double resultSummary = RasterBandAccessors.getSummaryStats(raster, "count", 1, false);
+        assertEquals(1036800, (int) resultSummary);
+
+        resultSummary = RasterBandAccessors.getSummaryStats(raster, "sum", 1, false);
+        assertEquals(207319567, (int) resultSummary);
+
+        resultSummary = RasterBandAccessors.getSummaryStats(raster, "mean", 1, false);
+        assertEquals(199, (int) resultSummary);
+
+        resultSummary = RasterBandAccessors.getSummaryStats(raster, "stddev", 1, false);
+        assertEquals(92, (int) resultSummary);
+
+        resultSummary = RasterBandAccessors.getSummaryStats(raster, "min", 1, false);
+        assertEquals(1, (int) resultSummary);
+
+        resultSummary = RasterBandAccessors.getSummaryStats(raster, "max", 1, false);
+        assertEquals(255, (int) resultSummary);
+    }
+
     @Test
     public void testSetBandNoDataValueWithReplaceOptionRaster() throws IOException {
         GridCoverage2D raster = rasterFromGeoTiff(resourceFolder + "raster/raster_with_no_data/test5.tiff");
-        double[] originalSummary = RasterBandAccessors.getSummaryStats(raster, 1, false);
+        double[] originalSummary = RasterBandAccessors.getSummaryStatsAll(raster, 1, false);
         int sumOG = (int) originalSummary[1];
 
         assertEquals(206233487, sumOG);
         GridCoverage2D resultRaster = RasterBandEditors.setBandNoDataValue(raster, 1, 10.0, true);
-        double[] resultSummary = RasterBandAccessors.getSummaryStats(resultRaster, 1, false);
+        double[] resultSummary = RasterBandAccessors.getSummaryStatsAll(resultRaster, 1, false);
         int sumActual = (int) resultSummary[1];
 
         // 108608 is the total no-data values in the raster
@@ -82,7 +109,7 @@ public class RasterBandEditorsTest extends RasterTestBase{
 
         // Not replacing previous no-data value
         resultRaster = RasterBandEditors.setBandNoDataValue(raster, 1, 10.0);
-        resultSummary = RasterBandAccessors.getSummaryStats(resultRaster, 1, false);
+        resultSummary = RasterBandAccessors.getSummaryStatsAll(resultRaster, 1, false);
         sumActual = (int) resultSummary[1];
         assertEquals(sumOG, sumActual);
     }
@@ -164,7 +191,7 @@ public class RasterBandEditorsTest extends RasterTestBase{
     }
 
     @Test
-    public void testClip() throws IOException, FactoryException, TransformException, ParseException {
+    public void testClip() throws IOException, FactoryException, TransformException, ParseException, ClassNotFoundException {
         GridCoverage2D raster = rasterFromGeoTiff(resourceFolder + "raster_geotiff_color/FAA_UTM18N_NAD83.tif");
         String polygon = "POLYGON ((236722 4204770, 243900 4204770, 243900 4197590, 221170 4197590, 236722 4204770))";
         Geometry geom = Constructors.geomFromWKT(polygon, RasterAccessors.srid(raster));
@@ -191,6 +218,8 @@ public class RasterBandEditorsTest extends RasterTestBase{
         GridCoverage2D croppedRaster = RasterBandEditors.clip(raster, 1, geom, 200, true);
         assertEquals(0, croppedRaster.getRenderedImage().getMinX());
         assertEquals(0, croppedRaster.getRenderedImage().getMinY());
+        GridCoverage2D croppedRaster2 = Serde.deserialize(Serde.serialize(croppedRaster));
+        assertSameCoverage(croppedRaster, croppedRaster2);
         points = new ArrayList<>();
         points.add(Constructors.geomFromWKT("POINT(236842 4.20465e+06)", 26918));
         points.add(Constructors.geomFromWKT("POINT(236961 4.20453e+06)", 26918));
@@ -202,6 +231,87 @@ public class RasterBandEditorsTest extends RasterTestBase{
         assertTrue(Arrays.equals(expectedValues, actualValues));
     }
 
+    @Test
+    public void testRasterUnion() throws FactoryException {
+        double[][] rasterData1 = new double[][] {
+                {13, 80, 49, 15, 4, 46, 47, 94, 58, 37, 6, 22, 98, 26, 78, 66, 86, 79, 5, 65, 7, 12, 89, 67},
+                {37, 4, 5, 15, 60, 83, 24, 19, 23, 87, 98, 89, 59, 71, 42, 46, 0, 80, 27, 73, 66, 100, 78, 64},
+                {73, 39, 50, 13, 45, 21, 87, 38, 63, 22, 44, 6, 8, 24, 19, 10, 89, 3, 48, 28, 0, 71, 59, 11}
+        };
+        GridCoverage2D raster1 = RasterConstructors.makeNonEmptyRaster(3, "i", 4, 6, 1, -1, 1, -1, 0, 0, 0, rasterData1);
+
+        double[][] rasterData2 = new double[][] {
+                {35, 68, 56, 87, 49, 20, 73, 90, 45, 96, 52, 98, 2, 82, 88, 74, 77, 60, 5, 61, 81, 32, 9, 15},
+                {55, 49, 72, 10, 63, 94, 100, 83, 61, 47, 20, 15, 34, 46, 52, 11, 23, 98, 70, 67, 18, 39, 53, 91}
+        };
+        GridCoverage2D raster2 = RasterConstructors.makeNonEmptyRaster(2, "d", 4, 6, 1, -1, 1, -1, 0, 0, 0, rasterData2);
+
+        GridCoverage2D result = rasterUnion(raster1, raster2);
+        int actualNumBands = RasterAccessors.numBands(result);
+        int expectedNumBands = 5;
+        assertEquals(expectedNumBands, actualNumBands);
+
+        double[] actualBandValues = MapAlgebra.bandAsArray(result, 5);
+        double[] expectedBandValues = MapAlgebra.bandAsArray(raster2, 2);
+        assertArrayEquals(expectedBandValues, actualBandValues, 0.1d);
+
+        double[] actualMetadata = Arrays.stream(RasterAccessors.metadata(result), 0, 9).toArray();
+        double[] expectedMetadata = Arrays.stream(RasterAccessors.metadata(raster2), 0, 9).toArray();
+        assertArrayEquals(expectedMetadata, actualMetadata, 0.1d);
+
+        result = rasterUnion(raster1, raster2, raster1);
+        actualNumBands = RasterAccessors.numBands(result);
+        expectedNumBands = 8;
+        assertEquals(expectedNumBands, actualNumBands);
+
+        actualBandValues = MapAlgebra.bandAsArray(result, 7);
+        expectedBandValues = MapAlgebra.bandAsArray(raster1, 2);
+        assertArrayEquals(expectedBandValues, actualBandValues, 0.1d);
+
+        actualMetadata = Arrays.stream(RasterAccessors.metadata(result), 0, 9).toArray();
+        expectedMetadata = Arrays.stream(RasterAccessors.metadata(raster2), 0, 9).toArray();
+        assertArrayEquals(expectedMetadata, actualMetadata, 0.1d);
+
+        result = rasterUnion(raster1, raster2, raster1, raster2);
+        actualNumBands = RasterAccessors.numBands(result);
+        expectedNumBands = 10;
+        assertEquals(expectedNumBands, actualNumBands);
+
+        actualBandValues = MapAlgebra.bandAsArray(result, 9);
+        expectedBandValues = MapAlgebra.bandAsArray(raster2, 1);
+        assertArrayEquals(expectedBandValues, actualBandValues, 0.1d);
+
+        actualMetadata = Arrays.stream(RasterAccessors.metadata(result), 0, 9).toArray();
+        expectedMetadata = Arrays.stream(RasterAccessors.metadata(raster2), 0, 9).toArray();
+        assertArrayEquals(expectedMetadata, actualMetadata, 0.1d);
+
+        result = rasterUnion(raster1, raster2, raster1, raster2, raster1);
+        actualNumBands = RasterAccessors.numBands(result);
+        expectedNumBands = 13;
+        assertEquals(expectedNumBands, actualNumBands);
+
+        actualBandValues = MapAlgebra.bandAsArray(result, 13);
+        expectedBandValues = MapAlgebra.bandAsArray(raster1, 3);
+        assertArrayEquals(expectedBandValues, actualBandValues, 0.1d);
+
+        actualMetadata = Arrays.stream(RasterAccessors.metadata(result), 0, 9).toArray();
+        expectedMetadata = Arrays.stream(RasterAccessors.metadata(raster2), 0, 9).toArray();
+        assertArrayEquals(expectedMetadata, actualMetadata, 0.1d);
+
+        result = rasterUnion(raster1, raster2, raster1, raster2, raster1, raster2);
+        actualNumBands = RasterAccessors.numBands(result);
+        expectedNumBands = 15;
+        assertEquals(expectedNumBands, actualNumBands);
+
+        actualBandValues = MapAlgebra.bandAsArray(result, 14);
+        expectedBandValues = MapAlgebra.bandAsArray(raster2, 1);
+        assertArrayEquals(expectedBandValues, actualBandValues, 0.1d);
+
+        actualMetadata = Arrays.stream(RasterAccessors.metadata(result), 0, 9).toArray();
+        expectedMetadata = Arrays.stream(RasterAccessors.metadata(raster2), 0, 9).toArray();
+        assertArrayEquals(expectedMetadata, actualMetadata, 0.1d);
+    }
+
     @Test
     public void testAddBandWithEmptyRaster() throws FactoryException {
         double[][] rasterData1 = new double[][] {
