@@ -46,12 +46,14 @@ public class Catalog {
                 new Functions.ST_Azimuth(),
                 new Functions.ST_Boundary(),
                 new Functions.ST_Buffer(),
+                new Functions.ST_BestSRID(),
                 new Functions.ST_ClosestPoint(),
                 new Functions.ST_Centroid(),
                 new Functions.ST_Collect(),
                 new Functions.ST_CollectionExtract(),
                 new Functions.ST_ConcaveHull(),
                 new Functions.ST_ConvexHull(),
+                new Functions.ST_CrossesDateLine(),
                 new Functions.ST_Envelope(),
                 new Functions.ST_Difference(),
                 new Functions.ST_Dimension(),
@@ -108,6 +110,7 @@ public class Catalog {
                 new Functions.ST_SetSRID(),
                 new Functions.ST_SRID(),
                 new Functions.ST_IsClosed(),
+                new Functions.ST_IsPolygonCW(),
                 new Functions.ST_IsRing(),
                 new Functions.ST_IsSimple(),
                 new Functions.ST_IsValid(),
@@ -120,22 +123,30 @@ public class Catalog {
                 new Functions.ST_LineSubstring(),
                 new Functions.ST_MakeLine(),
                 new Functions.ST_Polygon(),
+                new Functions.ST_Polygonize(),
                 new Functions.ST_MakePolygon(),
                 new Functions.ST_MakeValid(),
                 new Functions.ST_MinimumBoundingCircle(),
                 new Functions.ST_MinimumBoundingRadius(),
                 new Functions.ST_Multi(),
                 new Functions.ST_StartPoint(),
+                new Functions.ST_ShiftLongitude(),
                 new Functions.ST_SimplifyPreserveTopology(),
                 new Functions.ST_Split(),
                 new Functions.ST_Subdivide(),
                 new Functions.ST_SymDifference(),
                 new Functions.ST_S2CellIDs(),
+                new Functions.ST_Snap(),
+                new Functions.ST_S2ToGeom(),
                 new Functions.ST_GeometricMedian(),
                 new Functions.ST_NumPoints(),
                 new Functions.ST_Force3D(),
+                new Functions.ST_ForcePolygonCW(),
                 new Functions.ST_NRings(),
+                new Functions.ST_IsPolygonCCW(),
+                new Functions.ST_ForcePolygonCCW(),
                 new Functions.ST_Translate(),
+                new Functions.ST_Union(),
                 new Functions.ST_VoronoiPolygons(),
                 new Functions.ST_FrechetDistance(),
                 new Functions.ST_Affine(),
