@@ -22,14 +22,12 @@ import org.geotools.referencing.CRS;
 import org.junit.Assert;
 import org.junit.BeforeClass;
 import org.junit.Test;
-import org.locationtech.jts.geom.Geometry;
-import org.locationtech.jts.geom.LineString;
-import org.locationtech.jts.geom.Point;
-import org.locationtech.jts.geom.Polygon;
+import org.locationtech.jts.geom.*;
 import org.locationtech.jts.io.ParseException;
 import org.locationtech.jts.operation.buffer.BufferParameters;
 import org.opengis.referencing.FactoryException;
 import org.opengis.referencing.crs.CoordinateReferenceSystem;
+import scala.collection.immutable.Stream;
 
 import java.util.Arrays;
 import java.util.List;
@@ -86,17 +84,58 @@ public class FunctionTest extends TestBase{
         Geometry result = (Geometry) first(bufferTable).getField(0);
         assert(result instanceof Polygon);
 
-        String actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ReducePrecision(ST_Buffer(ST_GeomFromWKT('LINESTRING(0 0, 50 70, 100 100)'), 10, 'side=left'), 4))")).getField(0);
+        String actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ReducePrecision(ST_Buffer(ST_GeomFromWKT('LINESTRING(0 0, 50 70, 100 100)'), 10, false, 'side=left'), 4))")).getField(0);
         String expected = "POLYGON ((50 70, 0 0, -8.1373 5.8124, 41.8627 75.8124, 43.2167 77.3476, 44.855 78.5749, 94.855 108.5749, 100 100, 50 70))";
         assertEquals(expected, actual);
 
-        actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ReducePrecision(ST_Buffer(ST_GeomFromWKT('LINESTRING(0 0, 50 70, 70 -3)'), 10, 'endcap=square'), 4))")).getField(0);
+        actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ReducePrecision(ST_Buffer(ST_GeomFromWKT('LINESTRING(0 0, 50 70, 70 -3)'), 10, false, 'endcap=square'), 4))")).getField(0);
         expected = "POLYGON ((43.2156 77.3465, 44.8523 78.5733, 46.7044 79.4413, 48.6944 79.9144, 50.739 79.9727, 52.7527 79.6137, 54.6512 78.8525, 56.3552 77.7209, 57.7932 76.2663, 58.9052 74.5495, 59.6446 72.6424, 79.6446 -0.3576, 82.2869 -10.0022, 62.9978 -15.2869, 45.9128 47.0733, 8.1373 -5.8124, 2.325 -13.9497, -13.9497 -2.325, 41.8627 75.8124, 43.2156 77.3465))";
         assertEquals(expected, actual);
 
-        actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ReducePrecision(ST_Buffer(ST_Point(100, 90), 200, 'quad_segs=4'), 4))")).getField(0);
+        actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ReducePrecision(ST_Buffer(ST_Point(100, 90), 200, false, 'quad_segs=4'), 4))")).getField(0);
         expected = "POLYGON ((284.7759 13.4633, 241.4214 -51.4214, 176.5367 -94.7759, 100 -110, 23.4633 -94.7759, -41.4214 -51.4214, -84.7759 13.4633, -100 90, -84.7759 166.5367, -41.4214 231.4214, 23.4633 274.7759, 100 290, 176.5367 274.7759, 241.4214 231.4214, 284.7759 166.5367, 300 90, 284.7759 13.4633))";
         assertEquals(expected, actual);
+
+        actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ReducePrecision(ST_Buffer(ST_GeomFromWKT('LINESTRING(0 0, 50 70, 70 -3)'), 10, true, 'endcap=square'), 4))")).getField(0);
+        expected = "POLYGON ((50 70, 50.0001 70, 70.0001 -3, 70.0001 -3.0001, 69.9999 -3.0001, 50 69.9999, 0.0001 0, 0 -0.0001, -0.0001 0, 49.9999 70, 50 70))";
+        assertEquals(expected, actual);
+
+        actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ReducePrecision(ST_Buffer(ST_GeomFromWKT('POLYGON((-120 30, -80 30, -80 50, -120 50, -120 30))'), 200, true, 'quad_segs=4'), 4))")).getField(0);
+        expected = "POLYGON ((-120.0018 50, -120.0017 50.0004, -120.0013 50.0008, -120.0007 50.0011, -120 50.0012, -80 50.0012, -79.9993 50.0011, -79.9987 50.0008, -79.9983 50.0004, -79.9982 50, -79.9982 30, -79.9983 29.9994, -79.9987 29.9989, -79.9993 29.9986, -80 29.9984, -120 29.9984, -120.0007 29.9986, -120.0013 29.9989, -120.0017 29.9994, -120.0018 30, -120.0018 50))";
+        assertEquals(expected, actual);
+    }
+
+    @Test
+    public void testBestSRID() {
+        Table table1 = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('POINT (160 40)') AS geom");
+        table1 = table1.select(call(Functions.ST_BestSRID.class.getSimpleName(), $("geom")));
+        int result = (int) first(table1).getField(0);
+        assertEquals(32657, result);
+
+        Table table2 = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('LINESTRING(-91.185 30.4505, -91.187 30.452, -91.189 30.4535)') AS geom");
+        table2 = table2.select(call(Functions.ST_BestSRID.class.getSimpleName(), $("geom")));
+        result = (int) first(table2).getField(0);
+        assertEquals(32615, result);
+
+        Table table3 = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('POLYGON((-120 30, -80 30, -80 50, -120 50, -120 30))') AS geom");
+        table3 = table3.select(call(Functions.ST_BestSRID.class.getSimpleName(), $("geom")));
+        result = (int) first(table3).getField(0);
+        assertEquals(3395, result);
+    }
+
+    @Test
+    public void testShiftLogitude() {
+        String actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ShiftLongitude(ST_GeomFromWKT('POLYGON((179 10, -179 10, -179 20, 179 20, 179 10))')))")).getField(0);
+        String expected = "POLYGON ((179 10, 181 10, 181 20, 179 20, 179 10))";
+        assertEquals(expected, actual);
+
+        actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ShiftLongitude(ST_GeomFromWKT('MULTIPOLYGON(((179 10, -179 10, -179 20, 179 20, 179 10)), ((-185 10, -185 20, -175 20, -175 10, -185 10)))')))")).getField(0);
+        expected = "MULTIPOLYGON (((179 10, 181 10, 181 20, 179 20, 179 10)), ((175 10, 175 20, 185 20, 185 10, 175 10)))";
+        assertEquals(expected, actual);
+
+        actual = (String) first(tableEnv.sqlQuery("SELECT ST_AsText(ST_ShiftLongitude(ST_GeomFromWKT('LINESTRING(179 10, 181 10)')))")).getField(0);
+        expected = "LINESTRING (179 10, -179 10)";
+        assertEquals(expected, actual);
     }
 
     @Test
@@ -106,6 +145,8 @@ public class FunctionTest extends TestBase{
         Geometry result = (Geometry) first(table).getField(0);
         assertEquals("POINT (160 40)", result.toString());
     }
+
+    @Test
     public void testCentroid() {
         Table polygonTable = tableEnv.sqlQuery("SELECT ST_GeomFromText('POLYGON ((2 2, 0 0, 2 0, 0 2, 2 2))') as geom");
         Table resultTable = polygonTable.select(call(Functions.ST_Centroid.class.getSimpleName(), $("geom")));
@@ -168,6 +209,33 @@ public class FunctionTest extends TestBase{
         assertEquals("POLYGON ((0 0, 1 2, 3 2, 5 0, 0 0))", result.toString());
     }
 
+    @Test
+    public void testCrossesDateLine() {
+        // Test line crossing the Date Line
+        Table table1 = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('LINESTRING(170 30, -170 30)') AS geom");
+        table1 = table1.select(call("ST_CrossesDateLine", $("geom")));
+        Boolean actual1 = (Boolean) first(table1).getField(0);
+        assertEquals(true, actual1);
+
+        // Test line not crossing the Date Line
+        Table table2 = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('LINESTRING(-120 30, -130 40)') AS geom");
+        table2 = table2.select(call("ST_CrossesDateLine", $("geom")));
+        Boolean actual2 = (Boolean) first(table2).getField(0);
+        assertEquals(false, actual2);
+
+        // Test polygon crossing the Date Line
+        Table table3 = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('POLYGON((175 10, -175 10, -175 -10, 175 -10, 175 10))') AS geom");
+        table3 = table3.select(call("ST_CrossesDateLine", $("geom")));
+        Boolean actual3 = (Boolean) first(table3).getField(0);
+        assertEquals(true, actual3);
+
+        // Test polygon not crossing the Date Line
+        Table table4 = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('POLYGON((-120 10, -130 10, -130 -10, -120 -10, -120 10))') AS geom");
+        table4 = table4.select(call("ST_CrossesDateLine", $("geom")));
+        Boolean actual4 = (Boolean) first(table4).getField(0);
+        assertEquals(false, actual4);
+    }
+
     @Test
     public void testDifference() {
         Table lineTable = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('LINESTRING(50 100, 50 200)') AS g1, ST_GeomFromWKT('LINESTRING(50 50, 50 150)') as g2");
@@ -240,6 +308,22 @@ public class FunctionTest extends TestBase{
         assertEquals("POINT (-13134586.72 3764623.35)", result);
     }
 
+    @Test
+    public void testUnion() {
+        Table polyTable = tableEnv.sqlQuery("select ST_GeomFromWKT('POLYGON ((-3 -3, 3 -3, 3 3, -3 3, -3 -3))') as a, ST_GeomFromWKT('POLYGON ((-2 1, 2 1, 2 4, -2 4, -2 1))') as b");
+        String actual = first(polyTable.select(call(Functions.ST_Union.class.getSimpleName(), $("a"), $("b")))).getField(0).toString();
+        String expected = "POLYGON ((2 3, 3 3, 3 -3, -3 -3, -3 3, -2 3, -2 4, 2 4, 2 3))";
+        assertEquals(expected, actual);
+    }
+
+    @Test
+    public void testUnionArrayVariant() {
+        Table polyTable = tableEnv.sqlQuery("SELECT ARRAY[ST_GeomFromWKT('POLYGON ((-3 -3, 3 -3, 3 3, -3 3, -3 -3))'), ST_GeomFromWKT('POLYGON ((-2 1, 2 1, 2 4, -2 4, -2 1))')] as polys");
+        String actual = first(polyTable.select(call(Functions.ST_Union.class.getSimpleName(), $("polys")))).getField(0).toString();
+        String expected = "POLYGON ((2 3, 3 3, 3 -3, -3 -3, -3 3, -2 3, -2 4, 2 4, 2 3))";
+        assertEquals(expected, actual);
+    }
+
     @Test
     public void testTransformWKT() throws FactoryException {
         Table pointTable = createPointTable_real(testDataSize);
@@ -851,6 +935,16 @@ public class FunctionTest extends TestBase{
         assertEquals(4236, result.getSRID());
     }
 
+    @Test
+    public void testPolygonize() {
+        Table table = tableEnv.sqlQuery("SELECT ST_GeomFromEWKT('GEOMETRYCOLLECTION (LINESTRING (180 40, 30 20, 20 90), LINESTRING (180 40, 160 160), LINESTRING (80 60, 120 130, 150 80), LINESTRING (80 60, 150 80), LINESTRING (20 90, 70 70, 80 130), LINESTRING (80 130, 160 160), LINESTRING (20 90, 20 160, 70 190), LINESTRING (70 190, 80 130), LINESTRING (70 190, 160 160))') AS geom");
+        table = table.select(call(Functions.ST_Polygonize.class.getSimpleName(), $("geom")));
+        Geometry result = (Geometry) first(table).getField(0);
+        result.normalize();
+        String expected = "GEOMETRYCOLLECTION (POLYGON ((20 90, 20 160, 70 190, 80 130, 70 70, 20 90)), POLYGON ((20 90, 70 70, 80 130, 160 160, 180 40, 30 20, 20 90), (80 60, 150 80, 120 130, 80 60)), POLYGON ((70 190, 160 160, 80 130, 70 190)), POLYGON ((80 60, 120 130, 150 80, 80 60)))";
+        assertEquals(expected, result.toString());
+    }
+
     @Test
     public void testMakePolygon() {
         Table table = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('LINESTRING (0 0, 1 0, 1 1, 0 0)') AS line");
@@ -905,6 +999,20 @@ public class FunctionTest extends TestBase{
         assertEquals(0.5, result.getRight(), 1e-6);
     }
 
+    @Test
+    public void testSnap() {
+        Table base = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('POLYGON((2.6 12.5, 2.6 20.0, 12.6 20.0, 12.6 12.5, 2.6 12.5 ))') AS poly, ST_GeomFromWKT('LINESTRING (0.5 10.7, 5.4 8.4, 10.1 10.0)') AS line");
+        Table table = base.select(call(Functions.ST_Snap.class.getSimpleName(), $("poly"), $("line"), 2.525).as("result"));
+        String actual = (String) first(table.select(call(Functions.ST_AsText.class.getSimpleName(), $("result")))).getField(0);
+        String expected = "POLYGON ((2.6 12.5, 2.6 20, 12.6 20, 12.6 12.5, 10.1 10, 2.6 12.5))";
+        assertEquals(expected, actual);
+
+        table = base.select(call(Functions.ST_Snap.class.getSimpleName(), $("poly"), $("line"), 3.125).as("result"));
+        actual = (String) first(table.select(call(Functions.ST_AsText.class.getSimpleName(), $("result")))).getField(0);
+        expected = "POLYGON ((0.5 10.7, 2.6 20, 12.6 20, 12.6 12.5, 10.1 10, 5.4 8.4, 0.5 10.7))";
+        assertEquals(expected, actual);
+    }
+
     @Test
     public void testMulti() {
         Table table = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('POINT (0 0)') AS geom");
@@ -984,6 +1092,16 @@ public class FunctionTest extends TestBase{
         assertEquals(2, first(joinCleanedTable).getField(1));
     }
 
+    @Test
+    public void testS2ToGeom() {
+        Table pointTable = tableEnv.sqlQuery("select ST_S2ToGeom(ST_S2CellIDs(ST_GeomFromWKT('POLYGON ((0.1 0.1, 0.5 0.1, 1 0.3, 1 1, 0.1 1, 0.1 0.1))'), 10))");
+        Geometry target = (Geometry) first(tableEnv.sqlQuery("select ST_GeomFromWKT('POLYGON ((0.1 0.1, 0.5 0.1, 1 0.3, 1 1, 0.1 1, 0.1 0.1))')")).getField(0);
+        Geometry[] actual = (Geometry[]) Objects.requireNonNull(first(pointTable).getField(0));
+        assertTrue(actual[0].intersects(target));
+        assertTrue(actual[20].intersects(target));
+        assertTrue(actual[100].intersects(target));
+    }
+
     @Test
     public void testH3CellIDs() {
         String initExplodeQuery = "SELECT id, geom, cell_tbl.cell from (VALUES %s) as raw_tbl(id, geom, cells) CROSS JOIN UNNEST(raw_tbl.cells) AS cell_tbl (cell)";
@@ -1033,9 +1151,12 @@ public class FunctionTest extends TestBase{
 
     @Test
     public void testH3ToGeom() {
-            Table pointTable = tableEnv.sqlQuery("select ST_H3ToGeom(ST_H3CellIDs(ST_GeomFromWKT('POINT(1 2)'), 8, true))");
-        Geometry exact = (Geometry) Objects.requireNonNull(first(pointTable).getField(0));
-        assertEquals(exact.getNumGeometries(), 1);
+        Table pointTable = tableEnv.sqlQuery("select ST_H3ToGeom(ST_H3CellIDs(ST_GeomFromWKT('POLYGON ((0.1 0.1, 0.5 0.1, 1 0.3, 1 1, 0.1 1, 0.1 0.1))'), 4, true))");
+        Geometry target = (Geometry) first(tableEnv.sqlQuery("select ST_GeomFromWKT('POLYGON ((0.1 0.1, 0.5 0.1, 1 0.3, 1 1, 0.1 1, 0.1 0.1))')")).getField(0);
+        Geometry[] actual = (Geometry[]) first(pointTable).getField(0);
+        assertTrue(actual[0].intersects(target));
+        assertTrue(actual[11].intersects(target));
+        assertTrue(actual[20].intersects(target));
     }
 
     @Test
@@ -1095,6 +1216,25 @@ public class FunctionTest extends TestBase{
         assertEquals(expectedDims, actual);
     }
 
+    @Test
+    public void testForcePolygonCW() {
+        Table polyTable = tableEnv.sqlQuery("SELECT ST_ForcePolygonCW(ST_GeomFromWKT('POLYGON ((20 35, 10 30, 10 10, 30 5, 45 20, 20 35),(30 20, 20 15, 20 25, 30 20))')) AS polyCW");
+        String actual = (String) first(polyTable.select(call(Functions.ST_AsText.class.getSimpleName(), $("polyCW")))).getField(0);
+        String expected = "POLYGON ((20 35, 45 20, 30 5, 10 10, 10 30, 20 35), (30 20, 20 25, 20 15, 30 20))";
+        assertEquals(expected, actual);
+    }
+
+    @Test
+    public void testIsPolygonCW() {
+        Table polyTable = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('POLYGON ((20 35, 10 30, 10 10, 30 5, 45 20, 20 35),(30 20, 20 15, 20 25, 30 20))') AS polyCCW");
+        boolean actual = (boolean) first(polyTable.select(call(Functions.ST_IsPolygonCW.class.getSimpleName(), $("polyCCW")))).getField(0);
+        assertFalse(actual);
+
+        polyTable = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('POLYGON ((20 35, 45 20, 30 5, 10 10, 10 30, 20 35), (30 20, 20 25, 20 15, 30 20))') AS polyCW");
+        actual = (boolean) first(polyTable.select(call(Functions.ST_IsPolygonCW.class.getSimpleName(), $("polyCW")))).getField(0);
+        assertTrue(actual);
+    }
+
     @Test
     public void testNRings() {
         Integer expected = 1;
@@ -1103,6 +1243,25 @@ public class FunctionTest extends TestBase{
         assertEquals(expected, actual);
     }
 
+    @Test
+    public void testForcePolygonCCW() {
+        Table polyTable = tableEnv.sqlQuery("SELECT ST_ForcePolygonCCW(ST_GeomFromWKT('POLYGON ((20 35, 45 20, 30 5, 10 10, 10 30, 20 35), (30 20, 20 25, 20 15, 30 20))')) AS polyCW");
+        String actual = (String) first(polyTable.select(call(Functions.ST_AsText.class.getSimpleName(), $("polyCW")))).getField(0);
+        String expected = "POLYGON ((20 35, 10 30, 10 10, 30 5, 45 20, 20 35), (30 20, 20 15, 20 25, 30 20))";
+        assertEquals(expected, actual);
+    }
+
+    @Test
+    public void testIsPolygonCCW() {
+        Table polyTable = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('POLYGON ((20 35, 10 30, 10 10, 30 5, 45 20, 20 35),(30 20, 20 15, 20 25, 30 20))') AS polyCCW");
+        boolean actual = (boolean) first(polyTable.select(call(Functions.ST_IsPolygonCCW.class.getSimpleName(), $("polyCCW")))).getField(0);
+        assertTrue(actual);
+
+        polyTable = tableEnv.sqlQuery("SELECT ST_GeomFromWKT('POLYGON ((20 35, 45 20, 30 5, 10 10, 10 30, 20 35), (30 20, 20 25, 20 15, 30 20))') AS polyCW");
+        actual = (boolean) first(polyTable.select(call(Functions.ST_IsPolygonCCW.class.getSimpleName(), $("polyCW")))).getField(0);
+        assertFalse(actual);
+    }
+
     @Test
     public void testTranslate() {
         Table polyTable = tableEnv.sqlQuery("SELECT ST_Translate(ST_GeomFromWKT('POLYGON ((1 0, 1 1, 2 1, 2 0, 1 0))'), 2, 5)" + "AS " + polygonColNames[0]);
