@@ -159,7 +159,7 @@ public class UDFs {
     }
 
     @UDFAnnotations.ParamMeta(argNames = {"geometry", "radius"})
-    public static byte[] ST_Buffer(byte[] geometry, double radius) {
+    public static byte[] ST_Buffer(byte[] geometry, double radius) throws IllegalArgumentException {
         return GeometrySerde.serialize(
                 Functions.buffer(
                         GeometrySerde.deserialize(geometry),
@@ -168,6 +168,45 @@ public class UDFs {
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geometry", "radius", "useSpheroid"})
+    public static byte[] ST_Buffer(byte[] geometry, double radius, boolean useSpheroid) throws IllegalArgumentException {
+        return GeometrySerde.serialize(
+                Functions.buffer(
+                        GeometrySerde.deserialize(geometry),
+                        radius,
+                        useSpheroid
+                )
+        );
+    }
+
+    @UDFAnnotations.ParamMeta(argNames = {"geometry", "radius", "useSpheroid", "parameters"})
+    public static byte[] ST_Buffer(byte[] geometry, double radius, boolean useSpheroid, String parameters) throws IllegalArgumentException {
+        return GeometrySerde.serialize(
+                Functions.buffer(
+                        GeometrySerde.deserialize(geometry),
+                        radius,
+                        useSpheroid,
+                        parameters
+                )
+        );
+    }
+
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"})
+    public static int ST_BestSRID(byte[] geometry) {
+        return Functions.bestSRID(
+                GeometrySerde.deserialize(geometry)
+        );
+    }
+
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"})
+    public static byte[] ST_ShiftLongitude(byte[] geometry) {
+        return GeometrySerde.serialize(
+                Functions.shiftLongitude(
+                        GeometrySerde.deserialize(geometry)
+                )
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"geometry"})
     public static byte[] ST_BuildArea(byte[] geometry) {
         return GeometrySerde.serialize(
@@ -280,6 +319,13 @@ public class UDFs {
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"})
+    public static boolean ST_CrossesDateLine(byte[] geometry) {
+        return Functions.crossesDateLine(
+                GeometrySerde.deserialize(geometry)
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"angleInRadian"})
     public static double ST_Degrees(double angleInRadian) {
         return Functions.degrees(angleInRadian);
@@ -550,6 +596,20 @@ public class UDFs {
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"})
+    public static boolean ST_IsPolygonCW(byte[] geom) {
+        return Functions.isPolygonCW(
+                GeometrySerde.deserialize(geom)
+        );
+    }
+
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"})
+    public static boolean ST_IsPolygonCCW(byte[] geom) {
+        return Functions.isPolygonCCW(
+                GeometrySerde.deserialize(geom)
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"geometry"})
     public static boolean ST_IsRing(byte[] geometry) {
         return Functions.isRing(
@@ -887,6 +947,15 @@ public class UDFs {
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"})
+    public static byte[] ST_Polygonize(byte[] geometry) {
+        return GeometrySerde.serialize(
+                Functions.polygonize(
+                        GeometrySerde.deserialize(geometry)
+                )
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"minX", "minY", "maxX", "maxY"})
     public static byte[] ST_PolygonFromEnvelope(double minX, double minY, double maxX, double maxY) {
         return GeometrySerde.serialize(
@@ -1023,6 +1092,17 @@ public class UDFs {
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"input", "reference", "tolerance"})
+    public static byte[] ST_Snap(byte[] input, byte[] reference, double tolerance) {
+        return GeometrySerde.serialize(
+                Functions.snap(
+                        GeometrySerde.deserialize(input),
+                        GeometrySerde.deserialize(reference),
+                        tolerance
+                )
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"geometry", "maxVertices"})
     public static byte[] ST_SubDivide(byte[] geometry, int maxVertices) {
         return GeometrySerde.serialize(
@@ -1250,6 +1330,24 @@ public class UDFs {
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geom"})
+    public static byte[] ST_ForcePolygonCW(byte[] geom) {
+        return GeometrySerde.serialize(
+                Functions.forcePolygonCW(
+                        GeometrySerde.deserialize(geom)
+                )
+        );
+    }
+
+    @UDFAnnotations.ParamMeta(argNames = {"geom"})
+    public static byte[] ST_ForcePolygonCCW(byte[] geom) {
+        return GeometrySerde.serialize(
+                Functions.forcePolygonCCW(
+                        GeometrySerde.deserialize(geom)
+                )
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"geom"})
     public static double ST_LengthSpheroid(byte[] geom) {
         return Spheroid.length(
