@@ -171,8 +171,24 @@ public class UDFsV2
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"}, argTypes = {"Geometry"})
+    public static int ST_BestSRID(String geometry) {
+        return Functions.bestSRID(
+                GeometrySerde.deserGeoJson(geometry)
+        );
+    }
+
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"}, argTypes = {"Geometry"})
+    public static String ST_ShiftLongitude(String geometry) {
+        return GeometrySerde.serGeoJson(
+                Functions.shiftLongitude(
+                        GeometrySerde.deserGeoJson(geometry)
+                )
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"geometry", "radius"}, argTypes = {"Geometry", "double"}, returnTypes = "Geometry")
-    public static String ST_Buffer(String geometry, double radius) {
+    public static String ST_Buffer(String geometry, double radius) throws IllegalArgumentException {
         return GeometrySerde.serGeoJson(
                 Functions.buffer(
                         GeometrySerde.deserGeoJson(geometry),
@@ -181,6 +197,29 @@ public class UDFsV2
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geometry", "radius", "useSpheroid"}, argTypes = {"Geometry", "double", "boolean"}, returnTypes = "Geometry")
+    public static String ST_Buffer(String geometry, double radius, boolean useSpheroid) throws IllegalArgumentException {
+        return GeometrySerde.serGeoJson(
+                Functions.buffer(
+                        GeometrySerde.deserGeoJson(geometry),
+                        radius,
+                        useSpheroid
+                )
+        );
+    }
+
+    @UDFAnnotations.ParamMeta(argNames = {"geometry", "radius", "useSpheroid", "parameters"}, argTypes = {"Geometry", "double", "boolean", "String"}, returnTypes = "Geometry")
+    public static String ST_Buffer(String geometry, double radius, boolean useSpheroid, String parameters) throws IllegalArgumentException {
+        return GeometrySerde.serGeoJson(
+                Functions.buffer(
+                        GeometrySerde.deserGeoJson(geometry),
+                        radius,
+                        useSpheroid,
+                        parameters
+                )
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"geometry"}, argTypes = {"Geometry"}, returnTypes = "Geometry")
     public static String ST_BuildArea(String geometry) {
         return GeometrySerde.serGeoJson(
@@ -294,6 +333,13 @@ public class UDFsV2
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"}, argTypes = {"Geometry"})
+    public static boolean ST_CrossesDateLine(String geometry) {
+        return Functions.crossesDateLine(
+                GeometrySerde.deserGeoJson(geometry)
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"leftGeometry", "rightGeometry"}, argTypes = {"Geometry", "Geometry"}, returnTypes = "Geometry")
     public static String ST_Difference(String leftGeometry, String rightGeometry) {
         return GeometrySerde.serGeoJson(
@@ -500,6 +546,20 @@ public class UDFsV2
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geom"}, argTypes = {"Geometry"})
+    public static boolean ST_IsPolygonCCW(String geom) {
+        return Functions.isPolygonCCW(
+                GeometrySerde.deserGeoJson(geom)
+        );
+    }
+
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"}, argTypes = {"Geometry"})
+    public static boolean ST_IsPolygonCW(String geom) {
+        return Functions.isPolygonCW(
+                GeometrySerde.deserGeoJson(geom)
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"geometry"}, argTypes = {"Geometry"})
     public static boolean ST_IsRing(String geometry) {
         return Functions.isRing(
@@ -750,6 +810,15 @@ public class UDFsV2
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geometry"}, argTypes = {"Geometry"}, returnTypes = "Geometry")
+    public static String ST_Polygonize(String geometry) {
+        return GeometrySerde.serGeoJson(
+                Functions.polygonize(
+                        GeometrySerde.deserGeoJson(geometry)
+                )
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"geometry", "precisionScale"}, argTypes = {"Geometry", "int"}, returnTypes = "Geometry")
     public static String ST_PrecisionReduce(String geometry, int precisionScale) {
         return GeometrySerde.serGeoJson(
@@ -872,6 +941,17 @@ public class UDFsV2
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"input", "reference", "tolerance"}, argTypes = {"Geometry", "Geometry", "double"}, returnTypes = "Geometry")
+    public static String ST_Snap(String input, String reference, double tolerance) {
+        return GeometrySerde.serGeoJson(
+                Functions.snap(
+                        GeometrySerde.deserGeoJson(input),
+                        GeometrySerde.deserGeoJson(reference),
+                        tolerance
+                )
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"geometry", "maxVertices"}, argTypes = {"Geometry", "int"}, returnTypes = "Geometry")
     public static String ST_SubDivide(String geometry, int maxVertices) {
         return GeometrySerde.serGeoJson(
@@ -1093,6 +1173,24 @@ public class UDFsV2
         );
     }
 
+    @UDFAnnotations.ParamMeta(argNames = {"geom"}, argTypes = {"Geometry"}, returnTypes = "Geometry")
+    public static String ST_ForcePolygonCW(String geom) {
+        return GeometrySerde.serGeoJson(
+                Functions.forcePolygonCW(
+                        GeometrySerde.deserGeoJson(geom)
+                )
+        );
+    }
+
+    @UDFAnnotations.ParamMeta(argNames = {"geom"}, argTypes = {"Geometry"}, returnTypes = "Geometry")
+    public static String ST_ForcePolygonCCW(String geom) {
+        return GeometrySerde.serGeoJson(
+                Functions.forcePolygonCCW(
+                        GeometrySerde.deserGeoJson(geom)
+                )
+        );
+    }
+
     @UDFAnnotations.ParamMeta(argNames = {"geom"}, argTypes = {"Geometry"})
     public static double ST_LengthSpheroid(String geom) {
         return Spheroid.length(
