@@ -67,6 +67,8 @@ object Catalog {
     function[ST_NPoints](),
     function[ST_NDims](),
     function[ST_Buffer](),
+    function[ST_BestSRID](),
+    function[ST_ShiftLongitude](),
     function[ST_Envelope](),
     function[ST_Length](),
     function[ST_Area](),
@@ -83,6 +85,7 @@ object Catalog {
     function[ST_Touches](),
     function[ST_Overlaps](),
     function[ST_Crosses](),
+    function[ST_CrossesDateLine](),
     function[ST_IsSimple](),
     function[ST_MakeValid](false),
     function[ST_SimplifyPreserveTopology](),
@@ -102,6 +105,7 @@ object Catalog {
     function[ST_Y](),
     function[ST_Z](),
     function[ST_StartPoint](),
+    function[ST_Snap](),
     function[ST_ClosestPoint](),
     function[ST_Boundary](),
     function[ST_MinimumBoundingRadius](),
@@ -122,7 +126,10 @@ object Catalog {
     function[ST_AddPoint](-1),
     function[ST_RemovePoint](-1),
     function[ST_SetPoint](),
+    function[ST_IsPolygonCW](),
     function[ST_IsRing](),
+    function[ST_IsPolygonCCW](),
+    function[ST_ForcePolygonCCW](),
     function[ST_FlipCoordinates](),
     function[ST_LineSubstring](),
     function[ST_LineInterpolatePoint](),
@@ -131,6 +138,7 @@ object Catalog {
     function[ST_SubDivide](),
     function[ST_MakeLine](),
     function[ST_Polygon](),
+    function[ST_Polygonize](),
     function[ST_MakePolygon](null),
     function[ST_GeoHash](),
     function[ST_GeomFromGeoHash](null),
@@ -141,6 +149,7 @@ object Catalog {
     function[ST_PointN](),
     function[ST_AsEWKT](),
     function[ST_Force_2D](),
+    function[ST_ForcePolygonCW](),
     function[ST_ZMax](),
     function[ST_ZMin](),
     function[ST_YMax](),
@@ -156,6 +165,7 @@ object Catalog {
     function[ST_MLineFromText](0),
     function[ST_Split](),
     function[ST_S2CellIDs](),
+    function[ST_S2ToGeom](),
     function[ST_GeometricMedian](1e-6, 1000, false),
     function[ST_DistanceSphere](),
     function[ST_DistanceSpheroid](),
@@ -197,12 +207,15 @@ object Catalog {
     function[RS_LogicalOver](),
     function[RS_Array](),
     function[RS_Normalize](),
+    function[RS_NormalizeAll](),
     function[RS_AddBandFromArray](),
     function[RS_BandAsArray](),
     function[RS_MapAlgebra](null),
     function[RS_FromArcInfoAsciiGrid](),
     function[RS_FromGeoTiff](),
     function[RS_MakeEmptyRaster](),
+    function[RS_MakeRaster](),
+    function[RS_MakeRasterForTesting](),
     function[RS_Tile](),
     function[RS_TileExplode](),
     function[RS_Envelope](),
@@ -211,12 +224,14 @@ object Catalog {
     function[RS_SetSRID](),
     function[RS_SetGeoReference](),
     function[RS_SetBandNoDataValue](),
+    function[RS_SetPixelType](),
     function[RS_SetValues](),
     function[RS_SetValue](),
     function[RS_SRID](),
     function[RS_Value](1),
     function[RS_Values](1),
     function[RS_Intersects](),
+    function[RS_Interpolate](),
     function[RS_AsGeoTiff](),
     function[RS_AsRaster](),
     function[RS_AsArcGrid](),
@@ -224,6 +239,7 @@ object Catalog {
     function[RS_AsPNG](),
     function[RS_Width](),
     function[RS_Height](),
+    function[RS_Union](),
     function[RS_UpperLeftX](),
     function[RS_UpperLeftY](),
     function[RS_ScaleX](),
@@ -243,6 +259,7 @@ object Catalog {
     function[RS_Clip](),
     function[RS_Band](),
     function[RS_AddBand](),
+    function[RS_SummaryStatsAll](),
     function[RS_SummaryStats](),
     function[RS_BandIsNoData](),
     function[RS_ConvexHull](),
@@ -262,6 +279,7 @@ object Catalog {
     function[RS_ZonalStats](),
     function[RS_ZonalStatsAll](),
     function[RS_Resample](),
+    function[RS_ReprojectMatch]("nearestneighbor"),
     function[RS_FromNetCDF](),
     function[RS_NetCDFInfo]()
   )
