@@ -19,7 +19,7 @@
 
 package org.apache.sedona.sql.utils
 
-import org.apache.sedona.common.raster.Serde
+import org.apache.sedona.common.raster.serde.Serde
 import org.geotools.coverage.grid.GridCoverage2D
 
 object RasterSerializer {
