@@ -121,6 +121,14 @@ case class ST_ConvexHull(inputExpressions: Seq[Expression])
   }
 }
 
+case class ST_CrossesDateLine(inputExpressions: Seq[Expression])
+  extends InferredExpression(Functions.crossesDateLine _) {
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
 /**
   * Return the number of Points in geometry.
   *
@@ -153,13 +161,28 @@ case class ST_NDims(inputExpressions: Seq[Expression])
   * @param inputExpressions
   */
 case class ST_Buffer(inputExpressions: Seq[Expression])
-  extends InferredExpression(inferrableFunction2(Functions.buffer), inferrableFunction3(Functions.buffer)) {
+  extends InferredExpression(inferrableFunction2(Functions.buffer), inferrableFunction3(Functions.buffer), inferrableFunction4(Functions.buffer)) {
 
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
   }
 }
 
+case class ST_BestSRID(inputExpressions: Seq[Expression])
+  extends InferredExpression(Functions.bestSRID _) {
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
+case class ST_ShiftLongitude(inputExpressions: Seq[Expression])
+  extends InferredExpression(Functions.shiftLongitude _) {
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
 
 /**
   * Return the bounding rectangle for a Geometry
@@ -422,6 +445,14 @@ case class ST_StartPoint(inputExpressions: Seq[Expression])
   }
 }
 
+case class ST_Snap(inputExpressions: Seq[Expression])
+  extends InferredExpression(Functions.snap _) {
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
 case class ST_Boundary(inputExpressions: Seq[Expression])
   extends InferredExpression(Functions.boundary _) {
 
@@ -622,6 +653,13 @@ case class ST_ClosestPoint(inputExpressions: Seq[Expression])
   }
 }
 
+case class ST_IsPolygonCW(inputExpressions: Seq[Expression])
+  extends InferredExpression(Functions.isPolygonCW _) {
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
 case class ST_IsRing(inputExpressions: Seq[Expression])
   extends InferredExpression(ST_IsRing.isRing _) {
 
@@ -717,6 +755,14 @@ case class ST_Polygon(inputExpressions: Seq[Expression])
   }
 }
 
+case class ST_Polygonize(inputExpressions: Seq[Expression])
+  extends InferredExpression(Functions.polygonize _) {
+
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
 case class ST_MakePolygon(inputExpressions: Seq[Expression])
   extends InferredExpression(InferrableFunction.allowRightNull(Functions.makePolygon)) {
 
@@ -765,7 +811,7 @@ case class ST_SymDifference(inputExpressions: Seq[Expression])
   * @param inputExpressions
   */
 case class ST_Union(inputExpressions: Seq[Expression])
-  extends InferredExpression(Functions.union _) {
+  extends InferredExpression(inferrableFunction2(Functions.union), inferrableFunction1(Functions.union)) {
 
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
@@ -961,6 +1007,13 @@ case class ST_S2CellIDs(inputExpressions: Seq[Expression])
   }
 }
 
+case class ST_S2ToGeom(inputExpressions: Seq[Expression])
+  extends InferredExpression(Functions.s2ToGeom _) {
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
 case class ST_H3CellIDs(inputExpressions: Seq[Expression])
   extends InferredExpression(Functions.h3CellIDs _) {
 
@@ -1063,6 +1116,13 @@ case class ST_Force3D(inputExpressions: Seq[Expression])
   }
 }
 
+case class ST_ForcePolygonCW(inputExpressions: Seq[Expression])
+  extends InferredExpression(Functions.forcePolygonCW _) {
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
 case class ST_NRings(inputExpressions: Seq[Expression])
   extends InferredExpression(Functions.nRings _) {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
@@ -1070,6 +1130,19 @@ case class ST_NRings(inputExpressions: Seq[Expression])
   }
 }
 
+case class ST_IsPolygonCCW(inputExpressions: Seq[Expression]) extends InferredExpression(Functions.isPolygonCCW _) {
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
+case class ST_ForcePolygonCCW(inputExpressions: Seq[Expression])
+  extends InferredExpression(Functions.forcePolygonCCW _) {
+  protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
+    copy(inputExpressions = newChildren)
+  }
+}
+
 case class ST_Translate(inputExpressions: Seq[Expression])
   extends InferredExpression(inferrableFunction4(Functions.translate)) with FoldableExpression {
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
