@@ -166,7 +166,6 @@ case class ST_Crosses(inputExpressions: Seq[Expression])
   }
 }
 
-
 /**
   * Test if leftGeometry overlaps rightGeometry
   *
@@ -254,7 +253,7 @@ case class ST_OrderingEquals(inputExpressions: Seq[Expression])
 }
 
 case class ST_DWithin(inputExpressions: Seq[Expression])
-  extends InferredExpression(Predicates.dWithin _) {
+  extends InferredExpression(inferrableFunction3(Predicates.dWithin), inferrableFunction4(Predicates.dWithin)) {
 
   protected def withNewChildrenInternal(newChildren: IndexedSeq[Expression]) = {
     copy(inputExpressions = newChildren)
