@@ -66,8 +66,16 @@ object st_functions extends DataFrameAPI {
 
   def ST_Buffer(geometry: Column, buffer: Column): Column = wrapExpression[ST_Buffer](geometry, buffer)
   def ST_Buffer(geometry: String, buffer: Double): Column = wrapExpression[ST_Buffer](geometry, buffer)
-  def ST_Buffer(geometry: Column, buffer: Column, parameters: Column): Column = wrapExpression[ST_Buffer](geometry, buffer, parameters)
-  def ST_Buffer(geometry: String, buffer: Double, parameters: String): Column = wrapExpression[ST_Buffer](geometry, buffer, parameters)
+  def ST_Buffer(geometry: Column, buffer: Column, useSpheroid: Column): Column = wrapExpression[ST_Buffer](geometry, buffer, useSpheroid)
+  def ST_Buffer(geometry: String, buffer: Double, useSpheroid: Boolean): Column = wrapExpression[ST_Buffer](geometry, buffer, useSpheroid)
+  def ST_Buffer(geometry: Column, buffer: Column, useSpheroid: Column, parameters: Column): Column = wrapExpression[ST_Buffer](geometry, buffer, useSpheroid, parameters)
+  def ST_Buffer(geometry: String, buffer: Double, useSpheroid: Boolean, parameters: String): Column = wrapExpression[ST_Buffer](geometry, buffer, useSpheroid, parameters)
+
+  def ST_BestSRID(geometry: Column): Column = wrapExpression[ST_BestSRID](geometry)
+  def ST_BestSRID(geometry: String): Column = wrapExpression[ST_BestSRID](geometry)
+
+  def ST_ShiftLongitude(geometry: Column): Column = wrapExpression[ST_ShiftLongitude](geometry)
+  def ST_ShiftLongitude(geometry: String): Column = wrapExpression[ST_ShiftLongitude](geometry)
 
   def ST_BuildArea(geometry: Column): Column = wrapExpression[ST_BuildArea](geometry)
   def ST_BuildArea(geometry: String): Column = wrapExpression[ST_BuildArea](geometry)
@@ -95,6 +103,9 @@ object st_functions extends DataFrameAPI {
   def ST_ConvexHull(geometry: Column): Column = wrapExpression[ST_ConvexHull](geometry)
   def ST_ConvexHull(geometry: String): Column = wrapExpression[ST_ConvexHull](geometry)
 
+  def ST_CrossesDateLine(a: Column): Column = wrapExpression[ST_CrossesDateLine](a)
+  def ST_CrossesDateLine(a: String): Column = wrapExpression[ST_CrossesDateLine](a)
+
   def ST_Difference(a: Column, b: Column): Column = wrapExpression[ST_Difference](a, b)
   def ST_Difference(a: String, b: String): Column = wrapExpression[ST_Difference](a, b)
 
@@ -162,6 +173,9 @@ object st_functions extends DataFrameAPI {
   def ST_IsEmpty(geometry: Column): Column = wrapExpression[ST_IsEmpty](geometry)
   def ST_IsEmpty(geometry: String): Column = wrapExpression[ST_IsEmpty](geometry)
 
+  def ST_IsPolygonCW(geometry: Column): Column = wrapExpression[ST_IsPolygonCW](geometry)
+  def ST_IsPolygonCW(geometry: String): Column = wrapExpression[ST_IsPolygonCW](geometry)
+
   def ST_IsRing(lineString: Column): Column = wrapExpression[ST_IsRing](lineString)
   def ST_IsRing(lineString: String): Column = wrapExpression[ST_IsRing](lineString)
 
@@ -206,6 +220,9 @@ object st_functions extends DataFrameAPI {
   def ST_Polygon(lineString: Column, srid: Column): Column = wrapExpression[ST_Polygon](lineString, srid)
   def ST_Polygon(lineString: String, srid: Integer): Column = wrapExpression[ST_Polygon](lineString, srid)
 
+  def ST_Polygonize(geoms: Column): Column = wrapExpression[ST_Polygonize](geoms)
+  def ST_Polygonize(geoms: String): Column = wrapExpression[ST_Polygonize](geoms)
+
   def ST_MakePolygon(lineString: Column): Column = wrapExpression[ST_MakePolygon](lineString, null)
   def ST_MakePolygon(lineString: String): Column = wrapExpression[ST_MakePolygon](lineString, null)
   def ST_MakePolygon(lineString: Column, holes: Column): Column = wrapExpression[ST_MakePolygon](lineString, holes)
@@ -224,6 +241,12 @@ object st_functions extends DataFrameAPI {
   def ST_MinimumBoundingRadius(geometry: Column): Column = wrapExpression[ST_MinimumBoundingRadius](geometry)
   def ST_MinimumBoundingRadius(geometry: String): Column = wrapExpression[ST_MinimumBoundingRadius](geometry)
 
+  def ST_IsPolygonCCW(geometry: Column): Column = wrapExpression[ST_IsPolygonCCW](geometry)
+  def ST_IsPolygonCCW(geometry: String): Column = wrapExpression[ST_IsPolygonCCW](geometry)
+
+  def ST_ForcePolygonCCW(geometry: Column): Column = wrapExpression[ST_ForcePolygonCCW](geometry)
+  def ST_ForcePolygonCCW(geometry: String): Column = wrapExpression[ST_ForcePolygonCCW](geometry)
+
   def ST_Multi(geometry: Column): Column = wrapExpression[ST_Multi](geometry)
   def ST_Multi(geometry: String): Column = wrapExpression[ST_Multi](geometry)
 
@@ -261,6 +284,9 @@ object st_functions extends DataFrameAPI {
 
   def ST_S2CellIDs(geometry: String, level: Int): Column = wrapExpression[ST_S2CellIDs](geometry, level)
 
+  def ST_S2ToGeom(cellIDs: Column): Column = wrapExpression[ST_S2ToGeom](cellIDs)
+  def ST_S2ToGeom(cellIDs: Array[Long]): Column = wrapExpression[ST_S2ToGeom](cellIDs)
+
   def ST_SetPoint(lineString: Column, index: Column, point: Column): Column = wrapExpression[ST_SetPoint](lineString, index, point)
   def ST_SetPoint(lineString: String, index: Int, point: String): Column = wrapExpression[ST_SetPoint](lineString, index, point)
 
@@ -273,6 +299,9 @@ object st_functions extends DataFrameAPI {
   def ST_StartPoint(lineString: Column): Column = wrapExpression[ST_StartPoint](lineString)
   def ST_StartPoint(lineString: String): Column = wrapExpression[ST_StartPoint](lineString)
 
+  def ST_Snap(input: Column, reference: Column, tolerance: Column): Column = wrapExpression[ST_Snap](input, reference, tolerance)
+  def ST_Snap(input: String, reference: String, tolerance: Double): Column = wrapExpression[ST_Snap](input, reference, tolerance)
+
   def ST_SubDivide(geometry: Column, maxVertices: Column): Column = wrapExpression[ST_SubDivide](geometry, maxVertices)
   def ST_SubDivide(geometry: String, maxVertices: Int): Column = wrapExpression[ST_SubDivide](geometry, maxVertices)
 
@@ -297,6 +326,8 @@ object st_functions extends DataFrameAPI {
 
   def ST_Union(a: Column, b: Column): Column = wrapExpression[ST_Union](a, b)
   def ST_Union(a: String, b: String): Column = wrapExpression[ST_Union](a, b)
+  def ST_Union(geoms: Column): Column = wrapExpression[ST_Union](geoms)
+  def ST_Union(geoms: String): Column = wrapExpression[ST_Union](geoms)
 
   def ST_X(point: Column): Column = wrapExpression[ST_X](point)
   def ST_X(point: String): Column = wrapExpression[ST_X](point)
@@ -363,6 +394,9 @@ object st_functions extends DataFrameAPI {
 
   def ST_Force3D(geometry: String, zValue: Double): Column = wrapExpression[ST_Force3D](geometry, zValue)
 
+  def ST_ForcePolygonCW(geometry: Column): Column = wrapExpression[ST_ForcePolygonCW](geometry)
+  def ST_ForcePolygonCW(geometry: String): Column = wrapExpression[ST_ForcePolygonCW](geometry)
+
   def ST_NRings(geometry: Column): Column = wrapExpression[ST_NRings](geometry)
 
   def ST_NRings(geometry: String): Column = wrapExpression[ST_NRings](geometry)
