@@ -23,16 +23,17 @@ import org.apache.hadoop.fs.FileUtil
 import org.apache.hadoop.hdfs.{HdfsConfiguration, MiniDFSCluster}
 import org.apache.log4j.{Level, Logger}
 import org.apache.sedona.common.Functions.{frechetDistance, hausdorffDistance}
+import org.apache.sedona.common.Predicates.dWithin
 import org.apache.sedona.common.sphere.{Haversine, Spheroid}
 import org.apache.sedona.spark.SedonaContext
 import org.apache.spark.sql.DataFrame
-import org.locationtech.jts.geom.{CoordinateSequence, CoordinateSequenceComparator}
+import org.locationtech.jts.geom._
 import org.scalatest.{BeforeAndAfterAll, FunSpec}
 
 import java.io.File
 
 trait TestBaseScala extends FunSpec with BeforeAndAfterAll {
-  Logger.getRootLogger().setLevel(Level.WARN)
+  Logger.getRootLogger.setLevel(Level.WARN)
   Logger.getLogger("org.apache").setLevel(Level.WARN)
   Logger.getLogger("com").setLevel(Level.WARN)
   Logger.getLogger("akka").setLevel(Level.WARN)
@@ -76,7 +77,7 @@ trait TestBaseScala extends FunSpec with BeforeAndAfterAll {
   val rasterDataLocation: String = resourceFolder + "raster/raster_with_no_data/test5.tiff"
   val buildingDataLocation: String = resourceFolder + "813_buildings_test.csv"
   val smallRasterDataLocation: String = resourceFolder + "raster/test1.tiff"
-  var hdfsURI: String = _
+  private val factory = new GeometryFactory()
 
   override def beforeAll(): Unit = {
     SedonaContext.create(sparkSession)
@@ -99,6 +100,29 @@ trait TestBaseScala extends FunSpec with BeforeAndAfterAll {
     sparkSession.read.format("binaryFile").load(path)
   }
 
+
+  def generateTestData(): Seq[(Int, Double, Geometry)] = {
+    val geometries = (-180 to 180 by 10).flatMap { x =>
+      (-80 to 80 by 10).map { y =>
+        factory.createPoint(new Coordinate(x, y))
+      }
+    } ++ Seq(factory.createPoint(new Coordinate(0, -90)), factory.createPoint(new Coordinate(0, 90)))
+
+    // uniformly generate geometries.size partitions of distance between [110000,  110000 + 2000000)
+    geometries.zipWithIndex.map { case (geom, idx) =>
+      (idx, 110000 + 2000000 * (idx.asInstanceOf[Double] / geometries.size), geom)
+    }
+  }
+
+  def createSpheroidDataFrames(): Seq[DataFrame] = {
+    import sparkSession.implicits._
+    val sphericalTestData1 = generateTestData()
+    val sphericalTestData2 = generateTestData()
+    val sphericalDf1 = sphericalTestData1.toDF("id", "dist", "geom")
+    val sphericalDf2 = sphericalTestData2.toDF("id", "dist", "geom")
+    Seq(sphericalDf1, sphericalDf2)
+  }
+
   lazy val buildPointDf = loadCsv(csvPointInputLocation).selectExpr("ST_Point(cast(_c0 as Decimal(24,20)),cast(_c1 as Decimal(24,20))) as pointshape")
   lazy val buildPointLonLatDf = loadCsv(csvPointInputLocation).selectExpr("ST_Point(cast(_c1 as Decimal(24,20)),cast(_c0 as Decimal(24,20))) as pointshape")
   lazy val buildPolygonDf = loadCsv(csvPolygonInputLocation).selectExpr("ST_PolygonFromEnvelope(cast(_c0 as Decimal(24,20)),cast(_c1 as Decimal(24,20)), cast(_c2 as Decimal(24,20)), cast(_c3 as Decimal(24,20))) as polygonshape")
@@ -178,6 +202,32 @@ trait TestBaseScala extends FunSpec with BeforeAndAfterAll {
     }).sum
   }
 
+  protected def bruteForceDWithin(sampleCount: Int, distance: Double): Int = {
+    val inputPolygon = buildPolygonDf.limit(sampleCount).collect()
+    val inputPoint = buildPointDf.limit(sampleCount).collect()
+
+    inputPoint.map(row => {
+      val point = row.getAs[org.locationtech.jts.geom.Point](0)
+      inputPolygon.map(row => {
+        val polygon = row.getAs[org.locationtech.jts.geom.Polygon](0)
+        if (dWithin(point, polygon, distance, false)) 1 else 0
+      }).sum
+    }).sum
+  }
+
+  protected def bruteForceDWithinSphere(distance: Double): Int = {
+    val Seq(sphericalDf1, sphericalDf2) = createSpheroidDataFrames()
+    val sphericalDf1Collected = sphericalDf1.collect()
+    val sphericalDf2Collected = sphericalDf2.collect()
+    sphericalDf1Collected.map(row => {
+      val geom1 = row.get(2).asInstanceOf[org.locationtech.jts.geom.Point]
+      sphericalDf2Collected.map(row => {
+        val geom2 = row.get(2).asInstanceOf[org.locationtech.jts.geom.Point]
+        if (dWithin(geom1, geom2, distance, true)) 1 else 0
+      }).sum
+    }).sum
+  }
+
   /**
     * Create a mini HDFS cluster and return the HDFS instance and the URI.
     * @return (MiniDFSCluster, HDFS URI)
