@@ -19,12 +19,12 @@
 package org.apache.sedona.sql
 
 import org.apache.commons.codec.binary.Hex
-import org.apache.spark.sql.functions.{col, element_at, lit}
+import org.apache.spark.sql.functions.{array, col, element_at, lit}
 import org.apache.spark.sql.sedona_sql.expressions.st_aggregates._
 import org.apache.spark.sql.sedona_sql.expressions.st_constructors._
 import org.apache.spark.sql.sedona_sql.expressions.st_functions._
 import org.apache.spark.sql.sedona_sql.expressions.st_predicates._
-import org.junit.Assert.{assertEquals, assertTrue}
+import org.junit.Assert.{assertEquals, assertFalse, assertTrue}
 import org.locationtech.jts.geom.{Geometry, Polygon}
 import org.locationtech.jts.io.WKTWriter
 import org.locationtech.jts.operation.buffer.BufferParameters
@@ -198,22 +198,80 @@ class dataFrameAPITestScala extends TestBaseScala {
       assertEquals(expected, actual)
 
       var linestringDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING(0 0, 50 70, 100 100)') AS geom, 'side=left' as params")
-      var dfLine = linestringDf.select(ST_Buffer("geom", 10, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
+      var dfLine = linestringDf.select(ST_Buffer("geom", 10, false, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
       actual = dfLine.take(1)(0).get(0).asInstanceOf[Geometry].toText()
       expected = "POLYGON ((50 70, 0 0, -8.14 5.81, 41.86 75.81, 43.22 77.35, 44.86 78.57, 94.86 108.57, 100 100, 50 70))"
       assertEquals(expected, actual)
 
       linestringDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING(0 0, 50 70, 70 -3)') AS geom, 'endcap=square' AS params")
-      dfLine = linestringDf.select(ST_Buffer("geom", 10, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
+      dfLine = linestringDf.select(ST_Buffer("geom", 10, false, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
       actual = dfLine.take(1)(0).get(0).asInstanceOf[Geometry].toText()
       expected = "POLYGON ((43.22 77.35, 44.85 78.57, 46.7 79.44, 48.69 79.91, 50.74 79.97, 52.75 79.61, 54.65 78.85, 56.36 77.72, 57.79 76.27, 58.91 74.55, 59.64 72.64, 79.64 -0.36, 82.29 -10, 63 -15.29, 45.91 47.07, 8.14 -5.81, 2.32 -13.95, -13.95 -2.32, 41.86 75.81, 43.22 77.35))"
       assertEquals(expected, actual)
 
-      val pointDf = sparkSession.sql("SELECT ST_Point(100, 90) AS geom, 'quad_segs=4' as params")
-      val dfPoint = pointDf.select(ST_Buffer("geom", 200, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
+      linestringDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING(0 0, 50 70, 70 -3)') AS geom, 'endcap=square' AS params")
+      dfLine = linestringDf.select(ST_Buffer("geom", 10, true, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 4)")
+      actual = dfLine.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      expected = "POLYGON ((50 70, 50.0001 70, 70.0001 -3, 70.0001 -3.0001, 69.9999 -3.0001, 50 69.9999, 0.0001 0, 0 -0.0001, -0.0001 0, 49.9999 70, 50 70))"
+      assertEquals(expected, actual)
+
+      var pointDf = sparkSession.sql("SELECT ST_Point(100, 90) AS geom, 'quad_segs=4' as params")
+      var dfPoint = pointDf.select(ST_Buffer("geom", 200, false, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 2)")
       actual = dfPoint.take(1)(0).get(0).asInstanceOf[Geometry].toText()
       expected = "POLYGON ((284.78 13.46, 241.42 -51.42, 176.54 -94.78, 100 -110, 23.46 -94.78, -41.42 -51.42, -84.78 13.46, -100 90, -84.78 166.54, -41.42 231.42, 23.46 274.78, 100 290, 176.54 274.78, 241.42 231.42, 284.78 166.54, 300 90, 284.78 13.46))"
       assertEquals(expected, actual)
+
+      pointDf = sparkSession.sql("SELECT ST_Point(-180, 60) AS geom, 'quad_segs=4' as params")
+      dfPoint = pointDf.select(ST_Buffer("geom", 200, true, "params").as("geom")).selectExpr("ST_ReducePrecision(geom, 6)")
+      actual = dfPoint.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      expected = "POLYGON ((-179.99663 60.000611, -179.997353 60.001211, -179.998479 60.001626, -179.999837 60.001793, 179.99878 60.001688, 179.997583 60.001326, 179.996754 60.000761, 179.996419 60.000081, 179.99663 59.999389, 179.997353 59.998789, 179.99848 59.998374, 179.999837 59.998207, -179.99878 59.998312, -179.997583 59.998674, -179.996754 59.999238, -179.996419 59.999919, -179.99663 60.000611))"
+      assertEquals(expected, actual)
+    }
+
+    it("Passed ST_BestSRID") {
+      val pointDf = sparkSession.sql("SELECT ST_Point(-177, -60) AS geom")
+      val df = pointDf.select(ST_BestSRID("geom").as("geom"))
+      var actual = df.take(1)(0).get(0).asInstanceOf[Int]
+      var expected = 32701
+      assertEquals(expected, actual)
+
+      val linestringDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING(-91.185 30.4505, -91.187 30.452, -91.189 30.4535)') AS geom")
+      val dfLine = linestringDf.select(ST_BestSRID("geom").as("geom"))
+      actual = dfLine.take(1)(0).get(0).asInstanceOf[Int]
+      expected = 32615
+      assertEquals(expected, actual)
+
+      val polygonDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON((-120 30, -80 30, -80 50, -120 50, -120 30))') AS geom")
+      val dfPolygon = polygonDf.select(ST_BestSRID("geom").as("geom"))
+      actual = dfPolygon.take(1)(0).get(0).asInstanceOf[Int]
+      expected = 3395
+      assertEquals(expected, actual)
+
+      val polygonDf2 = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON((-73.9980 40.7265, -73.9970 40.7265, -73.9970 40.7255, -73.9980 40.7255, -73.9980 40.7265))') AS geom")
+      val dfPolygon2 = polygonDf2.select(ST_BestSRID("geom").as("geom"))
+      actual = dfPolygon2.take(1)(0).get(0).asInstanceOf[Int]
+      expected = 32618
+      assertEquals(expected, actual)
+    }
+
+    it("Passed ST_ShiftLongitude") {
+      val polygonDf = sparkSession.sql("SELECT ST_Point(1.0, 1.0) AS geom")
+      val df = polygonDf.select(ST_ShiftLongitude("geom").as("geom"))
+      var actual = df.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      var expected = "POINT (1 1)"
+      assertEquals(expected, actual)
+
+      var linestringDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING(179 75, 180 75, 181 75)') AS geom")
+      var dfLine = linestringDf.select(ST_ShiftLongitude("geom").as("geom"))
+      actual = dfLine.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      expected = "LINESTRING (179 75, 180 75, -179 75)"
+      assertEquals(expected, actual)
+
+      linestringDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING(0 10, -1 10)') AS geom")
+      dfLine = linestringDf.select(ST_ShiftLongitude("geom").as("geom"))
+      actual = dfLine.take(1)(0).get(0).asInstanceOf[Geometry].toText()
+      expected = "LINESTRING (0 10, 359 10)"
+      assertEquals(expected, actual)
     }
 
     it("Passed ST_Envelope") {
@@ -377,6 +435,15 @@ class dataFrameAPITestScala extends TestBaseScala {
       assert(actualResult.getSRID() == 4236)
     }
 
+    it("Passed ST_Polygonize") {
+      val invalidDf = sparkSession.sql("SELECT ST_GeomFromWKT('GEOMETRYCOLLECTION (LINESTRING (2 0, 2 1, 2 2), LINESTRING (2 2, 2 3, 2 4), LINESTRING (0 2, 1 2, 2 2), LINESTRING (2 2, 3 2, 4 2), LINESTRING (0 2, 1 3, 2 4), LINESTRING (2 4, 3 3, 4 2))') AS geom")
+      val df = invalidDf.select(ST_Polygonize("geom"))
+      val actualResult = df.take(1)(0).get(0).asInstanceOf[Geometry]
+      actualResult.normalize()
+      val expectedResult = "GEOMETRYCOLLECTION (POLYGON ((0 2, 1 3, 2 4, 2 3, 2 2, 1 2, 0 2)), POLYGON ((2 2, 2 3, 2 4, 3 3, 4 2, 3 2, 2 2)))"
+      assert(actualResult.toText()  == expectedResult)
+    }
+
     it("Passed `ST_MakePolygon`") {
       val invalidDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING (0 0, 1 0, 1 1, 0 0)') AS geom")
       val df = invalidDf.select(ST_MakePolygon("geom"))
@@ -505,6 +572,14 @@ class dataFrameAPITestScala extends TestBaseScala {
       assert(actualResult == expectedResult)
     }
 
+    it("Passed ST_Union array variant") {
+      val polygonDf = sparkSession.sql("SELECT array(ST_GeomFromWKT('POLYGON ((-3 -3, 3 -3, 3 3, -3 3, -3 -3))') , ST_GeomFromWKT('POLYGON ((-2 1, 2 1, 2 4, -2 4, -2 1))')) AS polys")
+      val df = polygonDf.select(ST_Union("polys"))
+      val actual = df.take(1)(0).get(0).asInstanceOf[Geometry].toText
+      val expected = "POLYGON ((2 3, 3 3, 3 -3, -3 -3, -3 3, -2 3, -2 4, 2 4, 2 3))"
+      assert(expected.equals(actual))
+    }
+
     it("Passed ST_Azimuth") {
       val baseDf = sparkSession.sql("SELECT ST_Point(0.0, 0.0) AS a, ST_Point(1.0, 1.0) AS b")
       val df = baseDf.select(ST_Azimuth("a", "b"))
@@ -545,6 +620,17 @@ class dataFrameAPITestScala extends TestBaseScala {
       assert(actualResult == expectedResult)
     }
 
+    it("Passed ST_Snap") {
+      val baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON((2.6 12.5, 2.6 20.0, 12.6 20.0, 12.6 12.5, 2.6 12.5 ))') AS poly, ST_GeomFromWKT('LINESTRING (0.5 10.7, 5.4 8.4, 10.1 10.0)') AS line")
+      var actual = baseDf.select(ST_Snap("poly", "line", 2.525)).take(1)(0).get(0).asInstanceOf[Geometry].toText
+      var expected = "POLYGON ((2.6 12.5, 2.6 20, 12.6 20, 12.6 12.5, 10.1 10, 2.6 12.5))"
+      assert(expected.equals(actual))
+
+      actual = baseDf.select(ST_Snap("poly", "line", 3.125)).take(1)(0).get(0).asInstanceOf[Geometry].toText
+      expected = "POLYGON ((0.5 10.7, 2.6 20, 12.6 20, 12.6 12.5, 10.1 10, 5.4 8.4, 0.5 10.7))"
+      assert(expected.equals(actual))
+    }
+
     it("Passed ST_Boundary") {
       val baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON ((0 0, 1 0, 1 1, 0 0))') AS geom")
       val df = baseDf.select(ST_Boundary("geom"))
@@ -553,6 +639,13 @@ class dataFrameAPITestScala extends TestBaseScala {
       assert(actualResult == expectedResult)
     }
 
+    it("Passed ST_CrossesDateLine") {
+      val baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON((175 10, -175 10, -175 -10, 175 -10, 175 10))') AS geom")
+      val df = baseDf.select(ST_CrossesDateLine("geom"))
+      val actualResult = df.take(1)(0).getBoolean(0)
+      assert(actualResult)
+    }
+
     it("Passed ST_EndPoint") {
       val baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('LINESTRING (0 0, 1 0)') AS geom")
       val df = baseDf.select(ST_EndPoint("geom"))
@@ -1014,9 +1107,20 @@ class dataFrameAPITestScala extends TestBaseScala {
       assert (actualResult.subsetOf(mbrResult))
     }
 
+    it("Passed ST_S2ToGeom") {
+      val baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON ((0.1 0.1, 0.5 0.1, 1 0.3, 1 1, 0.1 1, 0.1 0.1))') as geom")
+      val df = baseDf.select(ST_S2ToGeom(ST_S2CellIDs("geom", 10)).as("polygons"), col("geom"))
+      var intersectsDf = df.select(ST_Intersects(col("geom"), element_at(col("polygons"), 1)))
+      assert(intersectsDf.first().getBoolean(0))
+      intersectsDf = df.select(ST_Intersects(col("geom"), element_at(col("polygons"), 21)))
+      assert(intersectsDf.first().getBoolean(0))
+      intersectsDf = df.select(ST_Intersects(col("geom"), element_at(col("polygons"), 101)))
+      assert(intersectsDf.first().getBoolean(0))
+    }
+
     it("Passed ST_H3CellIDs") {
       val baseDF = sparkSession.sql("SELECT ST_GeomFromWKT('Polygon ((0 0, 1 2, 2 2, 3 2, 5 0, 4 0, 3 1, 2 1, 1 0, 0 0))') as geom")
-      val df = baseDF.select(ST_H3ToGeom(ST_H3CellIDs("geom", 6, true)))
+      val df = baseDF.select(ST_MakeValid(ST_Collect(ST_H3ToGeom(ST_H3CellIDs("geom", 6, true)))))
       val actualResult = df.take(1)(0).getAs[Geometry](0)
       val targetShape = baseDF.take(1)(0).getAs[Polygon](0);
       assert (actualResult.contains(targetShape))
@@ -1100,6 +1204,23 @@ class dataFrameAPITestScala extends TestBaseScala {
       assertEquals(expectedGeomDefaultValue, wktWriter.write(actualGeomDefaultValue))
     }
 
+    it("Passed ST_ForcePolygonCW") {
+      val baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON ((20 35, 10 30, 10 10, 30 5, 45 20, 20 35),(30 20, 20 15, 20 25, 30 20))') AS poly")
+      val actual = baseDf.select(ST_AsText(ST_ForcePolygonCW("poly"))).take(1)(0).get(0).asInstanceOf[String]
+      val expected = "POLYGON ((20 35, 45 20, 30 5, 10 10, 10 30, 20 35), (30 20, 20 25, 20 15, 30 20))"
+      assertEquals(expected, actual)
+    }
+
+    it("Passed ST_IsPolygonCW") {
+      var baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON ((20 35, 10 30, 10 10, 30 5, 45 20, 20 35),(30 20, 20 15, 20 25, 30 20))') as poly")
+      var actual = baseDf.select(ST_IsPolygonCW("poly")).take(1)(0).get(0).asInstanceOf[Boolean]
+      assertFalse(actual)
+
+      baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON ((20 35, 45 20, 30 5, 10 10, 10 30, 20 35), (30 20, 20 25, 20 15, 30 20))') as poly")
+      actual = baseDf.select(ST_IsPolygonCW("poly")).take(1)(0).get(0).asInstanceOf[Boolean]
+      assertTrue(actual)
+    }
+
     it("Passed ST_NRings") {
       val polyDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON ((1 0, 1 1, 2 1, 2 0, 1 0))') AS geom")
       val expected = 1
@@ -1108,6 +1229,23 @@ class dataFrameAPITestScala extends TestBaseScala {
       assert(expected == actual)
     }
 
+    it("Passed ST_ForcePolygonCCW") {
+      val baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON ((20 35, 45 20, 30 5, 10 10, 10 30, 20 35), (30 20, 20 25, 20 15, 30 20))') AS poly")
+      val actual = baseDf.select(ST_AsText(ST_ForcePolygonCCW("poly"))).take(1)(0).get(0).asInstanceOf[String]
+      val expected = "POLYGON ((20 35, 10 30, 10 10, 30 5, 45 20, 20 35), (30 20, 20 15, 20 25, 30 20))"
+      assertEquals(expected, actual)
+    }
+
+    it("Passed ST_IsPolygonCCW") {
+      var baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON ((20 35, 10 30, 10 10, 30 5, 45 20, 20 35),(30 20, 20 15, 20 25, 30 20))') as poly")
+      var actual = baseDf.select(ST_IsPolygonCCW("poly")).take(1)(0).get(0).asInstanceOf[Boolean]
+      assertTrue(actual)
+
+      baseDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON ((20 35, 45 20, 30 5, 10 10, 10 30, 20 35), (30 20, 20 25, 20 15, 30 20))') as poly")
+      actual = baseDf.select(ST_IsPolygonCCW("poly")).take(1)(0).get(0).asInstanceOf[Boolean]
+      assertFalse(actual)
+    }
+
     it("Passed ST_Translate") {
       val polyDf = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON ((1 0 1, 1 1 1, 2 1 1, 2 0 1, 1 0 1))') AS geom")
       val df = polyDf.select(ST_Translate("geom", 2, 3, 1))
