@@ -20,7 +20,7 @@
 package org.apache.sedona.sql
 
 import org.apache.sedona.core.utils.SedonaConf
-import org.apache.spark.sql.Row
+import org.apache.spark.sql.{DataFrame, Row}
 import org.apache.spark.sql.functions.expr
 import org.apache.spark.sql.sedona_sql.strategy.join.{DistanceJoinExec, RangeJoinExec}
 import org.apache.spark.sql.types._
@@ -502,5 +502,46 @@ class predicateJoinTestScala extends TestBaseScala {
         assert(distanceDefaultIntersectsDF.count() == expectedNoIntersects)
       })
     }
+
+    it ("Passed ST_DWithin (useSpheroid = false) in a spatial join") {
+      val sampleCount = 200
+      val distanceCandidates = Seq(1, 2, 5, 10)
+      val inputPoint = buildPointDf.limit(sampleCount).repartition(5)
+      val inputPolygon = buildPolygonDf.limit(sampleCount).repartition(3)
+
+      distanceCandidates.foreach(distance => {
+        val expected = bruteForceDWithin(sampleCount, distance)
+
+        val dfWithinDfShortForm = inputPoint.alias("pointDf").join(inputPolygon.alias("polygonDf"), expr(s"ST_DWithin(pointDf.pointshape, polygonDf.polygonshape, $distance)"))
+        assert(dfWithinDfShortForm.count() == expected)
+        assert(dfWithinDfShortForm.queryExecution.sparkPlan.collect { case p: DistanceJoinExec => p }.size === 1)
+
+        val dWithinDf = inputPoint.alias("pointDf").join(inputPolygon.alias("polygonDf"), expr(s"ST_DWithin(pointDf.pointshape, polygonDf.polygonshape, $distance, false)"))
+
+        assert(dWithinDf.count() == expected)
+        assert(dWithinDf.queryExecution.sparkPlan.collect { case p: DistanceJoinExec => p }.size === 1)
+      })
+    }
+
+    it ("Passed ST_DWithin useSphere long form (useSpheroid = true) in a spatial join") {
+      val Seq(sphericalDf1, sphericalDf2) = createSpheroidDataFrames()
+      val distanceCandidates = Seq(110000, 1055000, 2000000)
+
+      distanceCandidates.foreach(distance => {
+        val expected = bruteForceDWithinSphere(distance)
+        val dWithinDf = sphericalDf1.alias("df1").join(sphericalDf2.alias("df2"), expr(s"ST_DWithin(df1.geom, df2.geom, $distance, true)"))
+
+        assert(dWithinDf.count() == expected)
+        assert(dWithinDf.queryExecution.sparkPlan.collect { case p: DistanceJoinExec => p }.size === 1)
+      })
+    }
+
+    it("Passed ST_DWithin complex boolean expression") {
+      val expected = 55
+      val df_point = sparkSession.range(10).withColumn("pt", expr("ST_Point(id, id)"))
+      val df_polygon = sparkSession.range(10).withColumn("poly", expr("ST_Point(id, id + 0.01)"))
+      val actual = df_point.alias("a").join(df_polygon.alias("b"), expr("ST_DWithin(pt, poly, 10000, a.`id` % 2 = 0)")).count()
+      assert(expected == actual)
+    }
   }
 }
